# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# This module_utils is PRIVATE and should only be used by this collection. Breaking changes can occur any time.

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


import traceback

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.dns.plugins.module_utils.argspec import (
    ArgumentSpec,
    ModuleOptionProvider,
)

from ansible_collections.community.dns.plugins.module_utils.conversion.base import (
    DNSConversionError,
)

from ansible_collections.community.dns.plugins.module_utils.conversion.converter import (
    RecordConverter,
)

from ansible_collections.community.dns.plugins.module_utils.options import (
    create_record_transformation_argspec,
)

from ansible_collections.community.dns.plugins.module_utils.record import (
    format_record_for_output,
)

from ansible_collections.community.dns.plugins.module_utils.zone_record_api import (
    DNSAPIError,
    DNSAPIAuthenticationError,
    NOT_PROVIDED,
)

from ._utils import (
    normalize_dns_name,
    get_prefix,
)


def create_module_argument_spec(provider_information):
    return ArgumentSpec(
        argument_spec=dict(
            what=dict(type='str', choices=['single_record', 'all_types_for_record', 'all_records'], default='single_record'),
            zone_name=dict(type='str', aliases=['zone']),
            zone_id=dict(type=provider_information.get_zone_id_type()),
            record=dict(type='str'),
            prefix=dict(type='str'),
            type=dict(type='str', choices=provider_information.get_supported_record_types(), default=None),
        ),
        required_if=[
            ('what', 'single_record', ['type']),
            ('what', 'single_record', ['record', 'prefix'], True),
            ('what', 'all_types_for_record', ['record', 'prefix'], True),
        ],
        required_one_of=[
            ('zone_name', 'zone_id'),
        ],
        mutually_exclusive=[
            ('zone_name', 'zone_id'),
            ('record', 'prefix'),
        ],
    ).merge(create_record_transformation_argspec())


def run_module(module, create_api, provider_information):
    option_provider = ModuleOptionProvider(module)
    record_converter = RecordConverter(provider_information, option_provider)
    record_converter.emit_deprecations(module.deprecate)

    filter_record_type = NOT_PROVIDED
    filter_prefix = NOT_PROVIDED
    if module.params.get('what') == 'single_record':
        filter_record_type = module.params.get('type')
        if module.params.get('prefix') is not None:
            filter_prefix = provider_information.normalize_prefix(module.params.get('prefix'))
    elif module.params.get('what') == 'all_types_for_record':
        if module.params.get('prefix') is not None:
            filter_prefix = provider_information.normalize_prefix(module.params.get('prefix'))

    try:
        # Create API
        api = create_api()

        # Get zone information
        if module.params.get('zone_name') is not None:
            zone_in = normalize_dns_name(module.params.get('zone_name'))
            zone = api.get_zone_with_records_by_name(zone_in, prefix=filter_prefix, record_type=filter_record_type)
            if zone is None:
                module.fail_json(msg='Zone not found')
        else:
            zone = api.get_zone_with_records_by_id(module.params.get('zone_id'), prefix=filter_prefix, record_type=filter_record_type)
            if zone is None:
                module.fail_json(msg='Zone not found')
            zone_in = normalize_dns_name(zone.zone.name)

        # Retrieve requested information
        records = []
        if module.params.get('what') in ('single_record', 'all_types_for_record'):
            check_prefix = True
            record_in = normalize_dns_name(module.params.get('record'))
            prefix_in = module.params.get('prefix')
            record_in, prefix = get_prefix(
                normalized_zone=zone_in, normalized_record=record_in, prefix=prefix_in, provider_information=provider_information)
        else:
            check_prefix = False
            prefix = None

        # Find matching records
        records = []
        for record in zone.records:
            if check_prefix:
                if record.prefix != prefix:
                    continue
            records.append((
                (record.prefix + '.' + zone_in) if record.prefix else zone_in,
                record,
            ))

        # Convert records
        only_records = [record for record_name, record in records]
        record_converter.process_multiple_from_api(only_records)
        record_converter.process_multiple_to_user(only_records)

        # Format output
        data = [
            format_record_for_output(record, record_name, prefix=record.prefix)
            for record_name, record in records
        ]
        module.exit_json(
            changed=False,
            records=data,
            zone_id=zone.zone.id,
        )
    except DNSConversionError as e:
        module.fail_json(msg='Error while converting DNS values: {0}'.format(e.error_message), error=e.error_message, exception=traceback.format_exc())
    except DNSAPIAuthenticationError as e:
        module.fail_json(msg='Cannot authenticate: {0}'.format(e), error=to_text(e), exception=traceback.format_exc())
    except DNSAPIError as e:
        module.fail_json(msg='Error: {0}'.format(e), error=to_text(e), exception=traceback.format_exc())



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: guest_user
short_description: Resource module for Guest User
description:
- Manage operations create, update and delete of the resource Guest User.
- This API creates a guest user.
- This API deletes a guest user by ID.
- This API deletes a guest user.
- This API allows the client to update a guest user by ID.
- This API allows the client to update a guest user by name.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  customFields:
    description: Key value map.
    type: dict
  description:
    description: Guest User's description.
    type: str
  guestAccessInfo:
    description: Guest User's guestAccessInfo.
    suboptions:
      fromDate:
        description: Guest User's fromDate.
        type: str
      groupTag:
        description: Guest User's groupTag.
        type: str
      location:
        description: Guest User's location.
        type: str
      ssid:
        description: Guest User's ssid.
        type: str
      toDate:
        description: Guest User's toDate.
        type: str
      validDays:
        description: Guest User's validDays.
        type: int
    type: dict
  guestInfo:
    description: Guest User's guestInfo.
    suboptions:
      company:
        description: Guest User's company.
        type: str
      creationTime:
        description: Guest User's creationTime.
        type: str
      emailAddress:
        description: Guest User's emailAddress.
        type: str
      enabled:
        description: This field is only for Get operation not applicable for Create,
          Update operations.
        type: bool
      firstName:
        description: Guest User's firstName.
        type: str
      lastName:
        description: Guest User's lastName.
        type: str
      notificationLanguage:
        description: Guest User's notificationLanguage.
        type: str
      password:
        description: Guest User's password.
        type: str
      phoneNumber:
        description: Phone number should be E.164 format.
        type: str
      smsServiceProvider:
        description: Guest User's smsServiceProvider.
        type: str
      userName:
        description: If account needs be created with mobile number, please provide
          mobile number here.
        type: str
    type: dict
  guestType:
    description: Guest User's guestType.
    type: str
  id:
    description: Guest User's id.
    type: str
  name:
    description: Guest User's name.
    type: str
  portalId:
    description: Guest User's portalId.
    type: str
  reasonForVisit:
    description: Guest User's reasonForVisit.
    type: str
  sponsorUserId:
    description: Guest User's sponsorUserId.
    type: str
  sponsorUserName:
    description: Guest User's sponsorUserName.
    type: str
  status:
    description: Guest User's status.
    type: str
  statusReason:
    description: Guest User's statusReason.
    type: str
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
notes:
  - SDK Method used are
    guest_user.GuestUser.create_guest_user,
    guest_user.GuestUser.delete_guest_user_by_id,
    guest_user.GuestUser.delete_guest_user_by_name,
    guest_user.GuestUser.update_guest_user_by_id,
    guest_user.GuestUser.update_guest_user_by_name,

  - Paths used are
    post /ers/config/guestuser,
    delete /ers/config/guestuser/name/{name},
    delete /ers/config/guestuser/{id},
    put /ers/config/guestuser/name/{name},
    put /ers/config/guestuser/{id},

"""

EXAMPLES = r"""
- name: Update by name
  cisco.ise.guest_user:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    customFields: {}
    description: string
    guestAccessInfo:
      fromDate: string
      groupTag: string
      location: string
      ssid: string
      toDate: string
      validDays: 0
    guestInfo:
      company: string
      creationTime: string
      emailAddress: string
      enabled: true
      firstName: string
      lastName: string
      notificationLanguage: string
      password: string
      phoneNumber: string
      smsServiceProvider: string
      userName: string
    guestType: string
    id: string
    name: string
    portalId: string
    reasonForVisit: string
    sponsorUserId: string
    sponsorUserName: string
    status: string
    statusReason: string

- name: Delete by name
  cisco.ise.guest_user:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    name: string

- name: Update by id
  cisco.ise.guest_user:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    customFields: {}
    description: string
    guestAccessInfo:
      fromDate: string
      groupTag: string
      location: string
      ssid: string
      toDate: string
      validDays: 0
    guestInfo:
      company: string
      creationTime: string
      emailAddress: string
      enabled: true
      firstName: string
      lastName: string
      notificationLanguage: string
      password: string
      phoneNumber: string
      smsServiceProvider: string
      userName: string
    guestType: string
    id: string
    name: string
    portalId: string
    reasonForVisit: string
    sponsorUserId: string
    sponsorUserName: string
    status: string
    statusReason: string

- name: Delete by id
  cisco.ise.guest_user:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.guest_user:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    customFields: {}
    description: string
    guestAccessInfo:
      fromDate: string
      groupTag: string
      location: string
      ssid: string
      toDate: string
      validDays: 0
    guestInfo:
      company: string
      creationTime: string
      emailAddress: string
      enabled: true
      firstName: string
      lastName: string
      notificationLanguage: string
      password: string
      phoneNumber: string
      smsServiceProvider: string
      userName: string
    guestType: string
    name: string
    portalId: string
    reasonForVisit: string
    sponsorUserId: string
    sponsorUserName: string
    status: string
    statusReason: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "guestType": "string",
      "status": "string",
      "statusReason": "string",
      "reasonForVisit": "string",
      "sponsorUserId": "string",
      "sponsorUserName": "string",
      "guestInfo": {
        "firstName": "string",
        "lastName": "string",
        "company": "string",
        "creationTime": "string",
        "notificationLanguage": "string",
        "userName": "string",
        "emailAddress": "string",
        "phoneNumber": "string",
        "password": "string",
        "enabled": true,
        "smsServiceProvider": "string"
      },
      "guestAccessInfo": {
        "validDays": 0,
        "fromDate": "string",
        "toDate": "string",
        "location": "string",
        "ssid": "string",
        "groupTag": "string"
      },
      "portalId": "string",
      "customFields": {},
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The nxos_interfaces class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_diff,
    remove_empties,
    to_list,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    default_intf_enabled,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.utils.utils import (
    normalize_interface,
    search_obj_in_list,
)


class Interfaces(ConfigBase):
    """
    The nxos_interfaces class
    """

    gather_subset = ["min"]

    gather_network_resources = ["interfaces"]

    exclude_params = ["description", "mtu", "speed", "duplex"]

    def __init__(self, module):
        super(Interfaces, self).__init__(module)

    def get_interfaces_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :data: Mocked running-config data for state `parsed`
        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        self.facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )
        interfaces_facts = self.facts["ansible_network_resources"].get("interfaces")

        return interfaces_facts

    def get_platform(self):
        """Wrapper method for getting platform info
        This method exists solely to allow the unit test framework to mock calls.
        """
        return self.facts.get("ansible_net_platform", "")

    def get_system_defaults(self):
        """Wrapper method for `_connection.get()`
        This method exists solely to allow the unit test framework to mock device connection calls.
        """
        return self._connection.get(
            "show running-config all | incl 'system default switchport'",
        )

    def edit_config(self, commands):
        """Wrapper method for `_connection.edit_config()`
        This method exists solely to allow the unit test framework to mcdock device connection calls.
        """
        return self._connection.edit_config(commands)

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        commands = []
        warnings = []

        if self.state in self.ACTION_STATES:
            existing_interfaces_facts = self.get_interfaces_facts()
        else:
            existing_interfaces_facts = []

        if self.state in self.ACTION_STATES:
            self.intf_defs = self.render_interface_defaults(
                self.get_system_defaults(),
                existing_interfaces_facts,
            )
            commands.extend(self.set_config(existing_interfaces_facts))

        if self.state == "rendered":
            # Hardcode the system defaults for "rendered"
            # This can be made a configurable option in the future
            self.intf_defs = {
                "sysdefs": {
                    "L2_enabled": False,
                    "L3_enabled": False,
                    "mode": "layer3",
                },
            }
            commands.extend(self.set_config(existing_interfaces_facts))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self.edit_config(commands)
            result["changed"] = True

        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_interfaces_facts = self.get_interfaces_facts()

        elif self.state == "rendered":
            result["rendered"] = commands

        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"] = self.get_interfaces_facts(data=running_config)

        if self.state in self.ACTION_STATES:
            result["before"] = existing_interfaces_facts
            if result["changed"]:
                result["after"] = changed_interfaces_facts

        elif self.state == "gathered":
            result["gathered"] = changed_interfaces_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_interfaces_facts):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        config = self._module.params.get("config")
        want = []
        if config:
            for w in config:
                w.update({"name": normalize_interface(w["name"])})
                want.append(remove_empties(w))
        have = deepcopy(existing_interfaces_facts)
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params["state"]
        if state in ("overridden", "merged", "replaced", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(
                    state,
                ),
            )

        commands = list()
        if state == "overridden":
            commands.extend(self._state_overridden(want, have))
        elif state == "deleted":
            commands.extend(self._state_deleted(want, have))
        elif state == "purged":
            commands.extend(self._state_purged(want, have))
        else:
            for w in want:
                if state in ["merged", "rendered"]:
                    commands.extend(self._state_merged(w, have))
                elif state == "replaced":
                    commands.extend(self._state_replaced(w, have))
        return commands

    def _state_replaced(self, w, have):
        """The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        name = w["name"]
        obj_in_have = search_obj_in_list(name, have, "name")
        if obj_in_have:
            # If 'w' does not specify mode then intf may need to change to its
            # default mode, however default mode may depend on sysdef.
            if not w.get("mode") and re.search("Ethernet|port-channel", name):
                sysdefs = self.intf_defs["sysdefs"]
                sysdef_mode = sysdefs["mode"]
                if obj_in_have.get("mode") != sysdef_mode:
                    w["mode"] = sysdef_mode
            diff = dict_diff(w, obj_in_have)
        else:
            diff = w

        merged_commands = self.set_commands(w, have)
        # merged_commands:
        #   - These commands are changes specified by the playbook.
        #   - merged_commands apply to both existing and new objects
        # replaced_commands:
        #   - These are the unspecified commands, used to reset any params
        #     that are not already set to default states
        #   - replaced_commands should only be used on 'have' objects
        #     (interfaces that already exist)
        if obj_in_have:
            if "name" not in diff:
                diff["name"] = name
            wkeys = w.keys()
            dkeys = diff.keys()
            for k in wkeys:
                if k in self.exclude_params and k in dkeys:
                    del diff[k]
            replaced_commands = self.del_attribs(diff)
            cmds = set(replaced_commands).intersection(set(merged_commands))
            for cmd in cmds:
                merged_commands.remove(cmd)
            commands.extend(replaced_commands)

        commands.extend(merged_commands)
        return commands

    def _state_overridden(self, want, have):
        """The command generator when state is overridden

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        # overridden is the same as replaced behavior except for the scope.
        cmds = []
        existing_interfaces = []
        for h in have:
            existing_interfaces.append(h["name"])
            obj_in_want = search_obj_in_list(h["name"], want, "name")
            if obj_in_want:
                if h != obj_in_want:
                    replaced_cmds = self._state_replaced(obj_in_want, [h])
                    if replaced_cmds:
                        cmds.extend(replaced_cmds)
            else:
                cmds.extend(self.del_attribs(h))

        for w in want:
            if w["name"] not in existing_interfaces:
                # This is an object that was excluded from the 'have' list
                # because all of its params are currently set to default states
                # -OR- it's a new object that does not exist on the device yet.
                cmds.extend(self.add_commands(w))
        return cmds

    def _state_merged(self, w, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        return self.set_commands(w, have)

    def _state_deleted(self, want, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []
        if want:
            for w in want:
                obj_in_have = search_obj_in_list(w["name"], have, "name")
                commands.extend(self.del_attribs(obj_in_have))
        else:
            if not have:
                return commands
            for h in have:
                commands.extend(self.del_attribs(h))
        return commands

    def _state_purged(self, want, have):
        """The command generator when state is purged

        :rtype: A list
        :returns: the commands necessary to purge interfaces from running
                  configuration
        """
        commands = []
        if want:
            for w in want:
                obj_in_have = search_obj_in_list(w["name"], have, "name")
                if obj_in_have:
                    commands.append("no interface {0}".format(w["name"]))
        return commands

    def default_enabled(self, want=None, have=None, action=""):
        # 'enabled' default state depends on the interface type and L2 state.
        # Note that the current default could change when changing L2/L3 modes.
        if self.state == "rendered":
            # For "rendered", we always assume that
            # the default enabled state is False
            return False
        if want is None:
            want = {}
        if have is None:
            have = {}
        name = have.get("name")
        if name is None:
            return None

        sysdefs = self.intf_defs["sysdefs"]
        sysdef_mode = sysdefs["mode"]

        # Get the default enabled state for this interface. This was collected
        # during Facts gathering.
        intf_def_enabled = self.intf_defs.get(name)

        have_mode = have.get("mode", sysdef_mode)
        if action == "delete" and not want:
            want_mode = sysdef_mode
        else:
            want_mode = want.get("mode", have_mode)
        if (
            (want_mode and have_mode) is None
            or (want_mode != have_mode)
            or intf_def_enabled is None
        ):
            # L2-L3 is changing or this is a new virtual intf. Get new default.
            intf_def_enabled = default_intf_enabled(
                name=name,
                sysdefs=sysdefs,
                mode=want_mode,
            )
        return intf_def_enabled

    def del_attribs(self, obj):
        commands = []
        if not obj or len(obj.keys()) == 1:
            return commands
        # mode/switchport changes should occur before other changes
        sysdef_mode = self.intf_defs["sysdefs"]["mode"]
        if "mode" in obj and obj["mode"] != sysdef_mode:
            no_cmd = "no " if sysdef_mode == "layer3" else ""
            commands.append(no_cmd + "switchport")
        if "description" in obj:
            commands.append("no description")
        if "speed" in obj:
            commands.append("no speed")
        if "duplex" in obj:
            commands.append("no duplex")
        if "enabled" in obj:
            sysdef_enabled = self.default_enabled(have=obj, action="delete")
            if obj["enabled"] is False and sysdef_enabled is True:
                commands.append("no shutdown")
            elif obj["enabled"] is True and sysdef_enabled is False:
                commands.append("shutdown")
        if "mtu" in obj:
            commands.append("no mtu")
        if "ip_forward" in obj and obj["ip_forward"] is True:
            commands.append("no ip forward")
        if (
            "fabric_forwarding_anycast_gateway" in obj
            and obj["fabric_forwarding_anycast_gateway"] is True
        ):
            commands.append("no fabric forwarding mode anycast-gateway")
        if commands:
            commands.insert(0, "interface " + obj["name"])

        return commands

    def diff_of_dicts(self, w, obj):
        diff = set(w.items()) - set(obj.items())
        diff = dict(diff)
        if diff and w["name"] == obj["name"]:
            diff.update({"name": w["name"]})
        return diff

    def add_commands(self, d, obj_in_have=None):
        commands = []
        if obj_in_have is None:
            obj_in_have = {}
        # mode/switchport changes should occur before other changes
        if "mode" in d:
            sysdef_mode = self.intf_defs["sysdefs"]["mode"]
            have_mode = obj_in_have.get("mode", sysdef_mode)
            want_mode = d["mode"]
            if have_mode == "layer2":
                if want_mode == "layer3":
                    commands.append("no switchport")
            elif want_mode == "layer2":
                commands.append("switchport")
        if "description" in d:
            commands.append("description " + d["description"])
        if "speed" in d:
            commands.append("speed " + str(d["speed"]))
        if "duplex" in d:
            commands.append("duplex " + d["duplex"])
        if "enabled" in d:
            have_enabled = obj_in_have.get("enabled", self.default_enabled(d, obj_in_have)) or False
            if d["enabled"] is False and have_enabled is True:
                commands.append("shutdown")
            elif d["enabled"] is True and have_enabled is False:
                commands.append("no shutdown")
        if "mtu" in d or ("switchport" in commands):
            # changing mode to layer2 defaults the MTU
            # we need re-apply existing (non-default) MTU
            mtu = d.get("mtu") or obj_in_have.get("mtu")
            if mtu:
                commands.append("mtu " + str(mtu))
        if "ip_forward" in d:
            if d["ip_forward"] is True:
                commands.append("ip forward")
            else:
                commands.append("no ip forward")
        if "fabric_forwarding_anycast_gateway" in d:
            if d["fabric_forwarding_anycast_gateway"] is True:
                commands.append("fabric forwarding mode anycast-gateway")
            else:
                commands.append("no fabric forwarding mode anycast-gateway")
        if commands or not obj_in_have:
            commands.insert(0, "interface" + " " + d["name"])
        return commands

    def set_commands(self, w, have):
        commands = []
        obj_in_have = search_obj_in_list(w["name"], have, "name")
        if not obj_in_have:
            commands = self.add_commands(w)
        else:
            diff = self.diff_of_dicts(w, obj_in_have)
            commands = self.add_commands(diff, obj_in_have)
        return commands

    def render_interface_defaults(self, config, intfs):
        """Collect user-defined-default states for 'system default switchport'
        configurations. These configurations determine default L2/L3 modes
        and enabled/shutdown states. The default values for user-defined-default
        configurations may be different for legacy platforms.
        Notes:
        - L3 enabled default state is False on N9K,N7K but True for N3K,N6K
        - Changing L2-L3 modes may change the default enabled value.
        - '(no) system default switchport shutdown' only applies to L2 interfaces.
        Run through the gathered interfaces and tag their default enabled state.
        """
        intf_defs = {}
        L3_enabled = True if re.search("N[356]K", self.get_platform()) else False
        intf_defs = {
            "sysdefs": {
                "mode": None,
                "L2_enabled": None,
                "L3_enabled": L3_enabled,
            },
        }
        pat = "(no )*system default switchport$"
        m = re.search(pat, config, re.MULTILINE)
        if m:
            intf_defs["sysdefs"]["mode"] = "layer3" if "no " in m.groups() else "layer2"

        pat = "(no )*system default switchport shutdown$"
        m = re.search(pat, config, re.MULTILINE)
        if m:
            intf_defs["sysdefs"]["L2_enabled"] = True if "no " in m.groups() else False

        for item in intfs:
            intf_defs[item["name"]] = default_intf_enabled(
                name=item["name"],
                sysdefs=intf_defs["sysdefs"],
                mode=item.get("mode"),
            )

        return intf_defs



#!/usr/bin/python
# coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.0', 'status': ['preview'], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: project_update
author: "Sean Sullivan (@sean-m-sullivan)"
short_description: Update a Project in Automation Platform Controller
description:
    - Update a Automation Platform Controller Project. See
      U(https://www.ansible.com/tower) for an overview.
options:
    name:
      description:
        - The name or id of the project to update.
      required: True
      type: str
      aliases:
        - project
    organization:
      description:
        - Organization name, ID, or named URL the project exists in.
        - Used to help lookup the object, cannot be modified using this module.
        - If not provided, will lookup by name only, which does not work with duplicates.
      type: str
    wait:
      description:
        - Wait for the project to update.
        - If scm revision has not changed module will return not changed.
      default: True
      type: bool
    interval:
      description:
        - The interval to request an update from the controller.
      required: False
      default: 2
      type: float
    timeout:
      description:
        - If waiting for the project to update this will abort after this
          amount of seconds
      type: int
extends_documentation_fragment: awx.awx.auth
'''

RETURN = '''
id:
    description: project id of the updated project
    returned: success
    type: int
    sample: 86
status:
    description: status of the updated project
    returned: success
    type: str
    sample: pending
'''


EXAMPLES = '''
- name: Launch a project with a timeout of 10 seconds
  project_update:
    project: "Networking Project"
    timeout: 10

- name: Launch a Project with extra_vars without waiting
  project_update:
    project: "Networking Project"
    wait: False
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True, aliases=['project']),
        organization=dict(),
        wait=dict(default=True, type='bool'),
        interval=dict(default=2.0, type='float'),
        timeout=dict(type='int'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    name = module.params.get('name')
    organization = module.params.get('organization')
    wait = module.params.get('wait')
    interval = module.params.get('interval')
    timeout = module.params.get('timeout')

    # Attempt to look up project based on the provided name or id
    lookup_data = {}
    if organization:
        lookup_data['organization'] = module.resolve_name_to_id('organizations', organization)
    project = module.get_one('projects', name_or_id=name, data=lookup_data)
    if project is None:
        module.fail_json(msg="Unable to find project")

    if wait:
        scm_revision_original = project['scm_revision']

    # Update the project
    result = module.post_endpoint(project['related']['update'])

    if result['status_code'] == 405:
        module.fail_json(
            msg="Unable to trigger a project update because the project scm_type ({0}) does not support it.".format(project['scm_type']), response=result
        )
    elif result['status_code'] != 202:
        module.fail_json(msg="Failed to update project, see response for details", response=result)

    module.json_output['changed'] = True
    module.json_output['id'] = result['json']['id']
    module.json_output['status'] = result['json']['status']

    if not wait:
        module.exit_json(**module.json_output)

    # Invoke wait function
    result = module.wait_on_url(
        url=result['json']['url'], object_name=module.get_item_name(project), object_type='Project Update', timeout=timeout, interval=interval
    )
    scm_revision_new = result['json']['scm_revision']
    if scm_revision_new == scm_revision_original:
        module.json_output['changed'] = False

    module.exit_json(**module.json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_devtestlabartifact_info
version_added: "0.1.2"
short_description: Get Azure DevTest Lab Artifact facts
description:
    - Get facts of Azure DevTest Lab Artifact.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    lab_name:
        description:
            - The name of the lab.
        required: True
        type: str
    artifact_source_name:
        description:
            - The name of the artifact source.
        required: True
        type: str
    name:
        description:
            - The name of the artifact.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Get instance of DevTest Lab Artifact
  azure_rm_devtestlabartifact_info:
    resource_group: myResourceGroup
    lab_name: myLab
    artifact_source_name: myArtifactSource
    name: myArtifact
'''

RETURN = '''
artifacts:
    description:
        - A list of dictionaries containing facts for DevTest Lab Artifact.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The identifier of the artifact.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.DevTestLab/labs/myLab/ar
                     tifactSources/myArtifactSource/artifacts/myArtifact"
        resource_group:
            description:
                - Name of the resource group.
            returned: always
            type: str
            sample: myResourceGroup
        lab_name:
            description:
                - Name of the lab.
            returned: always
            type: str
            sample: myLab
        artifact_source_name:
            description:
                - The name of the artifact source.
            returned: always
            type: str
            sample: myArtifactSource
        name:
            description:
                - The name of the artifact.
            returned: always
            type: str
            sample: myArtifact
        description:
            description:
                - Description of the artifact.
            returned: always
            type: str
            sample: Installs My Software
        file_path:
            description:
                - Artifact's path in the repo.
            returned: always
            type: str
            sample: Artifacts/myArtifact
        publisher:
            description:
                - Publisher name.
            returned: always
            type: str
            sample: MyPublisher
        target_os_type:
            description:
                - Target OS type.
            returned: always
            type: str
            sample: Linux
        title:
            description:
                - Title of the artifact.
            returned: always
            type: str
            sample: My Software
        parameters:
            description:
                - A dictionary containing parameters definition of the artifact.
            returned: always
            type: dict
            sample: {}
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.devtestlabs import DevTestLabsClient
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMArtifactInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            lab_name=dict(
                type='str',
                required=True
            ),
            artifact_source_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.mgmt_client = None
        self.resource_group = None
        self.lab_name = None
        self.artifact_source_name = None
        self.name = None
        super(AzureRMArtifactInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])
        self.mgmt_client = self.get_mgmt_svc_client(DevTestLabsClient,
                                                    is_track2=True,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        if self.name:
            self.results['artifacts'] = self.get()
        else:
            self.results['artifacts'] = self.list()

        return self.results

    def get(self):
        response = None
        results = []
        try:
            response = self.mgmt_client.artifacts.get(resource_group_name=self.resource_group,
                                                      lab_name=self.lab_name,
                                                      artifact_source_name=self.artifact_source_name,
                                                      name=self.name)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError as e:
            self.log('Could not get facts for Artifact.')

        if response:
            results.append(self.format_response(response))

        return results

    def list(self):
        response = None
        results = []
        try:
            response = self.mgmt_client.artifacts.list(resource_group_name=self.resource_group,
                                                       lab_name=self.lab_name,
                                                       artifact_source_name=self.artifact_source_name)
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log('Could not get facts for Artifact.')

        if response is not None:
            for item in response:
                results.append(self.format_response(item))

        return results

    def format_response(self, item):
        d = item.as_dict()
        d = {
            'resource_group': self.parse_resource_to_dict(d.get('id')).get('resource_group'),
            'lab_name': self.parse_resource_to_dict(d.get('id')).get('name'),
            'artifact_source_name': self.parse_resource_to_dict(d.get('id')).get('child_name_1'),
            'id': d.get('id'),
            'description': d.get('description'),
            'file_path': d.get('file_path'),
            'name': d.get('name'),
            'parameters': d.get('parameters'),
            'publisher': d.get('publisher'),
            'target_os_type': d.get('target_os_type'),
            'title': d.get('title')
        }
        return d


def main():
    AzureRMArtifactInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Sudhakar Shet Kudtarkar (@kudtarkar1)
# Copyright: (c) 2020, Lionel Hercot <lhercot@cisco.com>
# Copyright: (c) 2020, Shreyas Srish <ssrish@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: aci_static_node_mgmt_address
short_description: In band or Out of band management IP address
description:
- Cisco ACI Fabric Node IP address
options:
  epg:
    description:
    - The name of the end point group
    type: str
  pod_id:
    description:
    - The pod number of the leaf, spine or APIC
    type: int
  node_id:
    description:
    - ACI Fabric's node id of a leaf, spine or APIC
    type: int
  ipv4_address:
    description:
    - ipv4 address of in band/out of band mgmt
    type: str
    aliases: [ ip ]
  ipv4_gw:
    description:
    - Gateway address of in band / out of band mgmt network
    type: str
    aliases: [ gw ]
  ipv6_address:
    description:
    -  ipv6 address of in band/out of band  mgmt
    type: str
    aliases: [ ipv6 ]
  ipv6_gw:
    description:
    - GW address of in band/out of band mgmt
    type: str
  type:
    description:
    - type of management interface
    type: str
    choices: [ in_band, out_of_band ]
    required: true
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

author:
- Sudhakar Shet Kudtarkar (@kudtarkar1)
- Lionel Hercot (@lhercot)
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Add ipv4 address to in band mgmt interface
  cisco.aci.aci_static_node_mgmt_address:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    epg: default
    pod_id: 1
    type: in_band
    node_id: 1102
    ipv4_address: "3.1.1.2/24"
    ipv4_gw: "3.1.1.1"
    state: present
  delegate_to: localhost

- name: Add ipv4 address to out of band mgmt interface
  cisco.aci.aci_static_node_mgmt_address:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    epg: default
    pod_id: 1
    type: out_of_band
    node_id: 1102
    ipv4_address: "3.1.1.2/24"
    ipv4_gw: "3.1.1.1"
    state: present
  delegate_to: localhost

- name: Remove ipv4 address to in band mgmt interface
  cisco.aci.aci_static_node_mgmt_address:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    epg: default
    pod_id: 1
    type: in_band
    node_id: 1102
    ipv4_address: "3.1.1.2/24"
    ipv4_gw: "3.1.1.1"
    state: absent
  delegate_to: localhost

- name: Query the in band mgmt ipv4 address
  cisco.aci.aci_static_node_mgmt_address:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    epg: default
    pod_id: 1
    type: in_band
    node_id: 1102
    ipv4_address: "3.1.1.2/24"
    ipv4_gw: "3.1.1.1"
    state: query
  delegate_to: localhost

- name: Query all addresses in epg out of band25wf
  cisco.aci.aci_static_node_mgmt_address:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    epg: default
    type: out_of_band
    state: query
  delegate_to: localhost

- name: Query all in band addresses
  cisco.aci.aci_static_node_mgmt_address:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    type: in_band
    state: query
  delegate_to: localhost
"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: class_map (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        node_id=dict(type="int"),
        pod_id=dict(type="int"),
        type=dict(type="str", choices=["in_band", "out_of_band"], required=True),
        epg=dict(type="str"),
        ipv4_address=dict(type="str", aliases=["ip"]),
        ipv4_gw=dict(type="str", aliases=["gw"]),
        ipv6_address=dict(type="str", aliases=["ipv6"]),
        ipv6_gw=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[["state", "absent", ["node_id", "epg"]], ["state", "present", ["node_id", "epg", "ipv4_address", "ipv4_gw"]]],
    )

    pod_id = module.params.get("pod_id")
    node_id = module.params.get("node_id")
    type = module.params.get("type")
    epg = module.params.get("epg")
    ipv4_address = module.params.get("ipv4_address")
    ipv4_gw = module.params.get("ipv4_gw")
    ipv6_address = module.params.get("ipv6_address")
    ipv6_gw = module.params.get("ipv6_gw")
    state = module.params.get("state")

    class_map = dict(
        in_band=list([dict(aci_class="mgmtInb", aci_rn="inb-{0}"), dict(aci_class="mgmtRsInBStNode", aci_rn="rsinBStNode-[{0}]")]),
        out_of_band=list([dict(aci_class="mgmtOob", aci_rn="oob-{0}"), dict(aci_class="mgmtRsOoBStNode", aci_rn="rsooBStNode-[{0}]")]),
    )

    static_path = None
    if pod_id is not None and node_id is not None:
        static_path = "topology/pod-{0}/node-{1}".format(pod_id, node_id)

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-mgmt",
            module_object="mgmt",
            target_filter={"name": "mgmt"},
        ),
        subclass_1=dict(
            aci_class="mgmtMgmtP",
            aci_rn="mgmtp-default",
            module_object="default",
            target_filter={"name": "default"},
        ),
        subclass_2=dict(
            aci_class=class_map.get(type)[0]["aci_class"],
            aci_rn=class_map.get(type)[0]["aci_rn"].format(epg),
            module_object=epg,
            target_filter={"name": epg},
        ),
        subclass_3=dict(
            aci_class=class_map.get(type)[1]["aci_class"],
            aci_rn=class_map.get(type)[1]["aci_rn"].format(static_path),
            module_object=static_path,
            target_filter={"name": static_path},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class=class_map.get(type)[1]["aci_class"],
            class_config=dict(addr=ipv4_address, gw=ipv4_gw, v6Addr=ipv6_address, v6Gw=ipv6_gw),
        )
        aci.get_diff(aci_class=class_map.get(type)[1]["aci_class"])

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_privatelinkservice_info

version_added: "1.12.0"

short_description: Get private endpoint connection info

description:
    - Get facts for private endpoint connection info.

options:
    name:
        description:
            - The name of the private link service.
        type: str
    resource_group:
        description:
            - The name of the resource group.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get private link service info by name
  azure_rm_privatelinkservice_info:
    name: pn-service-name
    resource_group: myResourceGroup

- name: Get all private link service by resource group
  azure_rm_privatelinkservice_info:
    resource_group: myResourceGroup

- name: Get all private link service by subscription filter by tags
  azure_rm_privatelinkservice_info:
    tags:
      - key1
      - abc
'''

RETURN = '''
link_service:
    description:
        - List of private link service info.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the private link service.
            sample: "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/privateLinkServices/linkservice"
            returned: always
            type: str
        name:
            description:
                - Name of the private link service.
            returned: always
            type: str
            sample: linkservice
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        alias:
            description:
                - The alias of the private link service.
            type: str
            returned: always
            sample: "linkservice.6a244dd8-8416-40cf-8c04-52b353bdd507.eastus.azure.privatelinkservice"
        auto_approval:
            description:
                - The auto-approval list of the private link service.
            type: dict
            returned: always
            sample: { "subscriptions": [] }
        enable_proxy_protocol:
            description:
                - Whether the private link service is enabled for proxy protocol or not
            type: bool
            returned: always
            sample: False
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            type: str
            returned: always
            sample: "f7d60f37-ea2b-4091-8546-1327f35468c4"
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: Microsoft.Network/privateLinkServices
        visibility:
            description:
                - The visibility list of the private link service.
            type: dict
            returned: always
            sample: { "subscriptions": [] }
        tags:
            description:
                - The resource tags.
            type: dict
            returned: always
            sample: { 'key1': 'value1' }
        provisioning_state:
            description:
                - The provisioning state of the private link service resource.
            type: str
            returned: always
            sample: Succeeded
        network_interfaces:
            description:
                - An array of references to the network interfaces created for this private link service.
            type: list
            returned: always
            sample:  [{ "id": "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/networkInterfaces/testlinkservice.nic.f5"}]
        ip_configurations:
            description:
                - An array of private link service IP configurations.
            type: complex
            returned: always
            contains:
                name:
                    description:
                        - The name of the IP configurations.
                    type: str
                    returned: always
                    sample: subnetfredprivate-1
                properties:
                    description:
                        - The IP configuration properties.
                    type: complex
                    returned: always
                    contains:
                        primary:
                            description:
                                - Whether the ip configuration is primary or not.
                            returned: always
                            type: bool
                            sample: True
                        private_ip_address_version:
                            description:
                                - Whether the specific IP configuration is IPv4 or IPv6.
                            returned: always
                            type: str
                            sample: IPv4
                        private_ip_allocation_method:
                            description:
                                - The private IP address allocation method.
                            returned: always
                            type: str
                            sample: Dynamic
                        subnet:
                            description:
                                - The reference to the subnet resource.
                            returned: always
                            type: dict
                            sample: { "id": "/subscriptions/xxx/resourceGroups/myRG/providers/Microsoft.Network/virtualNetworks/vnet/subnets/subname" }
        load_balancer_frontend_ip_configurations:
            description:
                - An array of references to the load balancer IP configurations.
            type: list
            returned: awalys
            sample: [{ "id": "/subscriptions/xxx/resourceGroups/myRG/providers/Microsoft.Network/loadBalancers/testlb/frontendIPConfigurations/front01" }]
        fqdns:
            description:
                - The list of Fqdn.
            type: list
            returned: always
            sample: ['fqdns1.com', 'fqdns2.com']
        private_endpoint_connections:
            description:
                - An array of list about connections to the private endpoint.
            type: complex
            returned: always
            contains:
                id:
                    description:
                        - The ID of the private endpoint connection.
                    type: str
                    returned: always
                    sample: "/subscriptions/xxx/resourceGroups/myReG/providers/Microsoft.Network/privateLinkServices/linkservice/privateEndpointConnections/tes"
                private_endpoint:
                    description:
                        - The ID of the private endpoint.
                    type: str
                    returned: always
                    sample: "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/privateEndpoints/test002"
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMPrivateLinkServiceInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type="str"),
            resource_group=dict(type="str"),
            tags=dict(type='list', elements='str')
        )

        self.name = None
        self.tags = None
        self.resource_group = None
        self.results = dict(
            changed=False,
        )

        super(AzureRMPrivateLinkServiceInfo, self).__init__(self.module_arg_spec,
                                                            supports_check_mode=True,
                                                            supports_tags=False,
                                                            facts_module=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name is not None and self.resource_group is not None:
            result = self.get_item()
        elif self.resource_group is not None:
            result = self.list_resourcegroup()
        else:
            result = self.list_by_subscription()

        self.results["link_service"] = [item for item in result if item and self.has_tags(item['tags'], self.tags)]
        return self.results

    def get_item(self):
        self.log("Get properties for {0} in {1}".format(self.name, self.resource_group))

        try:
            response = self.network_client.private_link_services.get(self.resource_group, self.name)
            return [self.service_to_dict(response)]
        except ResourceNotFoundError:
            self.log("Could not get info for {0} in {1}".format(self.name, self.resource_group))

        return []

    def list_resourcegroup(self):
        result = []
        self.log("List all in {0}".format(self.resource_group))
        try:
            response = self.network_client.private_link_services.list(self.resource_group)
            while True:
                result.append(response.next())
        except StopIteration:
            pass
        except Exception:
            pass
        return [self.service_to_dict(item) for item in result]

    def list_by_subscription(self):
        result = []
        self.log("List all in by subscription")
        try:
            response = self.network_client.private_link_services.list_by_subscription()
            while True:
                result.append(response.next())
        except StopIteration:
            pass
        except Exception:
            pass
        return [self.service_to_dict(item) for item in result]

    def service_to_dict(self, service_info):

        service = service_info.as_dict()
        result = dict(
            id=service.get("id"),
            name=service.get('name'),
            type=service.get('type'),
            etag=service.get('etag'),
            location=service.get('location'),
            tags=service.get('tags'),
            load_balancer_frontend_ip_configurations=service.get('load_balancer_frontend_ip_configurations'),
            ip_configurations=list(),
            network_interfaces=service.get('network_interfaces'),
            provisioning_state=service.get('provisioning_state'),
            private_endpoint_connections=list(),
            visibility=service.get('visibility'),
            auto_approval=service.get('auto_approval'),
            fqdns=service.get('fqdns'),
            alias=service.get('alias'),
            enable_proxy_protocol=service.get('enable_proxy_protocol')
        )
        if service.get('private_endpoint_connections'):
            for items in service['private_endpoint_connections']:
                result['private_endpoint_connections'].append({'id': items['id'], 'private_endpoint': items['private_endpoint']['id']})

        if service.get('ip_configurations'):
            for items in service['ip_configurations']:
                result['ip_configurations'].append(
                    {
                        "name": items['name'],
                        'properties': {
                            "primary": items['primary'],
                            "private_ip_address_version": items["private_ip_address_version"],
                            "private_ip_allocation_method": items["private_ip_allocation_method"],
                            "subnet": items["subnet"],
                        }
                    }
                )

        return result


def main():
    AzureRMPrivateLinkServiceInfo()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Felix Fontein
# Copyright (c) 2020 Markus Bergholz <markuman+spambelongstogoogle@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
name: hetzner_dns_records

short_description: Create inventory from Hetzner DNS records

version_added: 2.0.0

description:
    - This plugin allows to create an inventory from Hetzner DNS records.
    - >-
      For Ansible to be able to identify a YAML file as an inventory for this plugin, the inventory file must contain
      C(plugin: community.dns.hetzner_dns_records) and its filename must end with C(hetzner_dns.yaml) or C(hetzner_dns.yml)

options:
    plugin:
        description: The name of this plugin. Should always be set to V(community.dns.hetzner_dns_records) for this plugin to recognize it as its own.
        # TODO: add `required: true` in 3.0.0
        # required: true
        choices:
            - community.dns.hetzner_dns_records
        type: str

extends_documentation_fragment:
    - community.dns.hetzner
    - community.dns.hetzner.plugin
    - community.dns.hetzner.record_type_choices_records_inventory
    - community.dns.hetzner.zone_id_type
    - community.dns.inventory_records
    - community.dns.options.record_transformation

notes:
    - The provider-specific O(hetzner_token) option can be templated.

author:
    - Markus Bergholz (@markuman) <markuman+spambelongstogoogle@gmail.com>
    - Felix Fontein (@felixfontein)

seealso:
    - module: community.dns.hetzner_dns_record_set_info
    - module: community.dns.hetzner_dns_record_info
'''

EXAMPLES = '''
# filename must end with hetzner_dns.yaml or hetzner_dns.yml

plugin: community.dns.hetzner_dns_records
zone_name: domain.de
simple_filters:
  type:
    - TXT
txt_transformation: unquoted

# You can also configure the token by putting secret value into this file,
# but this is discouraged. Use a lookup like below, or leave it away and
# set it with the HETZNER_DNS_TOKEN environment variable.
hetzner_token: >-
    {{ (lookup('community.sops.sops', 'keys/hetzner.sops.yml') | from_yaml).hetzner_dns_token }}
'''

from ansible_collections.community.dns.plugins.module_utils.http import (
    OpenURLHelper,
)

from ansible_collections.community.dns.plugins.module_utils.hetzner.api import (
    create_hetzner_api,
    create_hetzner_provider_information,
)

from ansible_collections.community.dns.plugins.plugin_utils.templated_options import (
    TemplatedOptionProvider,
)

from ansible_collections.community.dns.plugins.plugin_utils.inventory.records import (
    RecordsInventoryModule,
)


class InventoryModule(RecordsInventoryModule):
    NAME = 'community.dns.hetzner_dns_records'
    VALID_ENDINGS = ('hetzner_dns.yaml', 'hetzner_dns.yml')

    def setup_api(self):
        self.provider_information = create_hetzner_provider_information()
        self.api = create_hetzner_api(TemplatedOptionProvider(self, self.templar), OpenURLHelper())



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_bastionhost_info

version_added: "1.13.0"

short_description: Get Azure bastion host info

description:
    - Get facts for Azure bastion host info.

options:
    name:
        description:
            - Name of the bastion host.
        type: str
    resource_group:
        description:
            - The name of the resource group.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get bastion host info by name
  azure_rm_bastionhost_info:
    name: bastion-name
    resource_group: myResourceGroup

- name: Get all bastion host by resource group
  azure_rm_bastionhost_info:
    resource_group: myResourceGroup

- name: Get all bastion hoste by subscription filter by tags
  azure_rm_bastionhost_info:
    tags:
      - key1
      - abc
'''

RETURN = '''
bastion_host:
    description:
        - List of Azure bastion host info.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the Azure bastion host.
            sample: "/subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/bastionHosts/testbastion"
            returned: always
            type: str
        name:
            description:
                - Name of the Azure bastion host.
            returned: always
            type: str
            sample: linkservice
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            type: str
            returned: always
            sample: "fb0e3a90-6afa-4a01-9171-9c84d144a0f3"
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: Microsoft.Network/bastionHosts
        tags:
            description:
                - The resource tags.
            type: list
            returned: always
            sample: { 'key1': 'value1' }
        provisioning_state:
            description:
                - The provisioning state of the bastion host resource.
            type: str
            returned: always
            sample: Succeeded
        scale_units:
            description:
                - The scale units for the Bastion Host resource.
            type: int
            returned: always
            sample: 2
        enable_tunneling:
            description:
                - Enable/Disable Tunneling feature of the Bastion Host resource.
            type: bool
            returned: always
            sample: False
        enable_shareable_link:
            description:
                - Enable/Disable Shareable Link of the Bastion Host resource.
            type: bool
            returned: always
            sample: False
        enable_ip_connect:
            description:
                - Enable/Disable IP Connect feature of the Bastion Host resource.
            type: bool
            returned: always
            sample: False
        enable_file_copy:
            description:
                - Enable/Disable File Copy feature of the Bastion Host resource.
            type: bool
            returned: always
            sample: False
        dns_name:
            description:
                - FQDN for the endpoint on which bastion host is accessible.
            type: str
            returned: always
            sample: bst-0ca1e1b6-9969-4167-be54-5972e1395c25.bastion.azure.com
        disable_copy_paste:
            description:
                - Enable/Disable Copy/Paste feature of the Bastion Host resource.
            type: bool
            returned: always
            sample: False
        sku:
            description:
                - The sku of this Bastion Host.
            type: complex
            returned: always
            contains:
                name:
                    description:
                        -  The name of this Bastion Host.
                    type: str
                    returned: always
                    sample: Standard
        ip_configurations:
            description:
                - An array of bastion host IP configurations.
            type: complex
            returned: always
            contains:
                name:
                    description:
                        - Name of the resource that is unique within a resource group.
                        - This name can be used to access the resource.
                    type: str
                    returned: always
                    sample: IpConf
                private_ip_allocation_method:
                    description:
                        - Private IP allocation method.
                    type: str
                    returned: always
                    sample: Static
                public_ip_address:
                    description:
                        - Reference of the PublicIP resource.
                    type: complex
                    returned: always
                    contains:
                        id:
                            description:
                                - The ID of the public IP address.
                            returned: always
                            type: str
                            sample: "/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.Network/publicIPAddresses/Myip"
                subnet:
                    description:
                        - The reference to the subnet resource.
                    returned: always
                    type: complex
                    contains:
                        id:
                            description:
                                - The ID of the subnet..
                            returned: always
                            type: str
                            sample:  "/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.Network/virtualNetworks/vnet/subnets/AzureBastionSubnet"
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMBastionHostInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type="str"),
            resource_group=dict(type="str"),
            tags=dict(type='list', elements='str')
        )

        self.name = None
        self.tags = None
        self.resource_group = None
        self.results = dict(
            changed=False,
        )

        super(AzureRMBastionHostInfo, self).__init__(self.module_arg_spec,
                                                     supports_check_mode=True,
                                                     supports_tags=False,
                                                     facts_module=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name is not None and self.resource_group is not None:
            result = self.get_item()
        elif self.resource_group is not None:
            result = self.list_resourcegroup()
        else:
            result = self.list_by_subscription()

        self.results["bastion_host"] = [item for item in result if item and self.has_tags(item['tags'], self.tags)]
        return self.results

    def get_item(self):
        self.log("Get properties for {0} in {1}".format(self.name, self.resource_group))

        try:
            response = self.network_client.bastion_hosts.get(self.resource_group, self.name)
            return [self.bastion_to_dict(response)]
        except ResourceNotFoundError:
            self.log("Could not get info for {0} in {1}".format(self.name, self.resource_group))

        return []

    def list_resourcegroup(self):
        result = []
        self.log("List all in {0}".format(self.resource_group))
        try:
            response = self.network_client.bastion_hosts.list_by_resource_group(self.resource_group)
            while True:
                result.append(response.next())
        except StopIteration:
            pass
        except Exception:
            pass
        return [self.bastion_to_dict(item) for item in result]

    def list_by_subscription(self):
        result = []
        self.log("List all in by subscription")
        try:
            response = self.network_client.bastion_hosts.list()
            while True:
                result.append(response.next())
        except StopIteration:
            pass
        except Exception:
            pass
        return [self.bastion_to_dict(item) for item in result]

    def bastion_to_dict(self, bastion_info):
        bastion = bastion_info.as_dict()
        result = dict(
            id=bastion.get("id"),
            name=bastion.get('name'),
            type=bastion.get('type'),
            etag=bastion.get('etag'),
            location=bastion.get('location'),
            tags=bastion.get('tags'),
            sku=dict(),
            ip_configurations=list(),
            dns_name=bastion.get('dns_name'),
            provisioning_state=bastion.get('provisioning_state'),
            scale_units=bastion.get('scale_units'),
            disable_copy_paste=bastion.get('disable_copy_paste'),
            enable_file_copy=bastion.get('enable_file_copy'),
            enable_ip_connect=bastion.get('enable_ip_connect'),
            enable_shareable_link=bastion.get('enable_tunneling'),
            enable_tunneling=bastion.get('enable_tunneling')
        )

        if bastion.get('sku'):
            result['sku']['name'] = bastion['sku']['name']

        if bastion.get('ip_configurations'):
            for items in bastion['ip_configurations']:
                result['ip_configurations'].append(
                    {
                        "name": items['name'],
                        "subnet": dict(id=items['subnet']['id']),
                        "public_ip_address": dict(id=items['public_ip_address']['id']),
                        "private_ip_allocation_method": items['private_ip_allocation_method'],
                    }
                )
        return result


def main():
    AzureRMBastionHostInfo()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_import_management
short_description: Import the primary Security Management Server database or the primary Multi-Domain Server database or the single Domain database and the
                   applicable Check Point configuration.
description:
  - Import the primary Security Management Server database or the primary Multi-Domain Server database or the single Domain database and the applicable
    Check Point configuration. <br/>After the import starts, the session expires and you must login again.
  - All operations are performed over Web Services API.
version_added: "5.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  file_path:
    description:
      - Path to the exported database file to be imported.
    type: str
    required: True
  domain_name:
    description:
      - Domain name to be imported. Must be unique in the Multi-Domain Server.<br><font color="red">Required only for</font> importing the Security
        Management Server into the Multi-Domain Server.
    type: str
  domain_ip_address:
    description:
      - IPv4 address for the imported Domain.<br><font color="red">Required only for</font> importing the Security Management Server into the
        Multi-Domain Server.
    type: str
  domain_server_name:
    description:
      - Multi-Domain Server name for the imported Domain.<br><font color="red">Required only for</font> importing the Security Management Server into
        the Multi-Domain Server.
    type: str
  include_logs:
    description:
      - Import logs without log indexes.
    type: bool
  include_logs_indexes:
    description:
      - Import logs with log indexes.
    type: bool
  keep_cloud_sharing:
    description:
      - Preserve the connection of the Management Server to Check Point's Infinity Portal.<br>Use this flag after ensuring that the original
        Management Server does not communicate with Infinity Portal.<br>Note, resuming the connection is also possible after import with set-cloud-services.
    type: bool
  include_endpoint_configuration:
    description:
      - Include import of the Endpoint Security Management configuration files.
    type: bool
  include_endpoint_database:
    description:
      - Include import of the Endpoint Security Management database.
    type: bool
  verify_domain_restore:
    description:
      - If true, verify that the restore operation is valid for this input file and this environment. <br>Note, Restore operation will not be executed.
    type: bool
  pre_import_verification_only:
    description:
      - If true, only runs the pre-import verifications instead of the full import.
    type: bool
  ignore_warnings:
    description:
      - Ignoring the verification warnings. By Setting this parameter to 'true' import will not be blocked by warnings.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: import-management
  cp_mgmt_import_management:
    file_path: /var/log/domain1_exported.tgz
"""

RETURN = """
cp_mgmt_import_management:
  description: The checkpoint import-management output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        file_path=dict(type='str', required=True),
        domain_name=dict(type='str'),
        domain_ip_address=dict(type='str'),
        domain_server_name=dict(type='str'),
        include_logs=dict(type='bool'),
        include_logs_indexes=dict(type='bool'),
        keep_cloud_sharing=dict(type='bool'),
        include_endpoint_configuration=dict(type='bool'),
        include_endpoint_database=dict(type='bool'),
        verify_domain_restore=dict(type='bool'),
        pre_import_verification_only=dict(type='bool'),
        ignore_warnings=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "import-management"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the ios_logging_global module
"""


class Logging_globalArgs(object):  # pylint: disable=R0903
    """The arg spec for the ios_logging_global module"""

    argument_spec = {
        "config": {
            "type": "dict",
            "options": {
                "buffered": {
                    "type": "dict",
                    "options": {
                        "size": {"type": "int"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                        "discriminator": {"type": "str"},
                        "filtered": {"type": "bool"},
                        "xml": {"type": "bool"},
                    },
                },
                "buginf": {"type": "bool"},
                "cns_events": {
                    "type": "str",
                    "choices": [
                        "alerts",
                        "critical",
                        "debugging",
                        "emergencies",
                        "errors",
                        "informational",
                        "notifications",
                        "warnings",
                    ],
                },
                "console": {
                    "type": "dict",
                    "options": {
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                                "guaranteed",
                            ],
                        },
                        "discriminator": {"type": "str"},
                        "filtered": {"type": "bool"},
                        "xml": {"type": "bool"},
                    },
                },
                "count": {"type": "bool"},
                "delimiter": {"type": "dict", "options": {"tcp": {"type": "bool"}}},
                "discriminator": {"type": "list", "elements": "str"},
                "dmvpn": {"type": "dict", "options": {"rate_limit": {"type": "int"}}},
                "esm": {"type": "dict", "options": {"config": {"type": "bool"}}},
                "exception": {"type": "int"},
                "facility": {
                    "type": "str",
                    "choices": [
                        "auth",
                        "cron",
                        "daemon",
                        "kern",
                        "local0",
                        "local1",
                        "local2",
                        "local3",
                        "local4",
                        "local5",
                        "local6",
                        "local7",
                        "lpr",
                        "mail",
                        "news",
                        "sys10",
                        "sys11",
                        "sys12",
                        "sys13",
                        "sys14",
                        "sys9",
                        "syslog",
                        "user",
                        "uucp",
                    ],
                },
                "filter": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "url": {"type": "str"},
                        "order": {"type": "int"},
                        "args": {"type": "str"},
                    },
                },
                "history": {
                    "type": "dict",
                    "options": {
                        "size": {"type": "int"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                    },
                },
                "hosts": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "discriminator": {"type": "str"},
                        "filtered": {"type": "bool"},
                        "sequence_num_session": {"type": "bool"},
                        "session_id": {
                            "type": "dict",
                            "options": {
                                "tag": {"type": "str", "choices": ["hostname", "ipv4", "ipv6"]},
                                "text": {"type": "str"},
                            },
                        },
                        "stream": {"type": "int"},
                        "transport": {
                            "type": "dict",
                            "options": {
                                "tcp": {
                                    "type": "dict",
                                    "options": {
                                        "audit": {"type": "bool"},
                                        "discriminator": {"type": "str"},
                                        "stream": {"type": "int"},
                                        "filtered": {"type": "bool"},
                                        "port": {"type": "int"},
                                        "sequence_num_session": {"type": "bool"},
                                        "session_id": {
                                            "type": "dict",
                                            "options": {
                                                "tag": {
                                                    "type": "str",
                                                    "choices": ["hostname", "ipv4", "ipv6"],
                                                },
                                                "text": {"type": "str"},
                                            },
                                        },
                                        "xml": {"type": "bool"},
                                    },
                                },
                                "udp": {
                                    "type": "dict",
                                    "options": {
                                        "discriminator": {"type": "str"},
                                        "stream": {"type": "int"},
                                        "filtered": {"type": "bool"},
                                        "port": {"type": "int"},
                                        "sequence_num_session": {"type": "bool"},
                                        "session_id": {
                                            "type": "dict",
                                            "options": {
                                                "tag": {
                                                    "type": "str",
                                                    "choices": ["hostname", "ipv4", "ipv6"],
                                                },
                                                "text": {"type": "str"},
                                            },
                                        },
                                        "xml": {"type": "bool"},
                                    },
                                },
                            },
                        },
                        "vrf": {"type": "str"},
                        "xml": {"type": "bool"},
                        "ipv6": {"type": "str"},
                        "host": {"type": "str", "aliases": ["hostname"]},
                    },
                },
                "message_counter": {
                    "type": "list",
                    "elements": "str",
                    "choices": ["log", "debug", "syslog"],
                },
                "monitor": {
                    "type": "dict",
                    "options": {
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                        "discriminator": {"type": "str"},
                        "filtered": {"type": "bool"},
                        "xml": {"type": "bool"},
                    },
                },
                "logging_on": {"type": "str", "choices": ["enable", "disable"]},
                "origin_id": {
                    "type": "dict",
                    "options": {
                        "tag": {"type": "str", "choices": ["hostname", "ip", "ipv6"]},
                        "text": {"type": "str"},
                    },
                },
                "persistent": {
                    "type": "dict",
                    "options": {
                        "batch": {"type": "int"},
                        "filesize": {"type": "int"},
                        "immediate": {"type": "bool"},
                        "notify": {"type": "bool"},
                        "protected": {"type": "bool"},
                        "size": {"type": "int"},
                        "threshold": {"type": "int"},
                        "url": {"type": "str"},
                    },
                },
                "policy_firewall": {"type": "dict", "options": {"rate_limit": {"type": "int"}}},
                "queue_limit": {
                    "type": "dict",
                    "options": {
                        "size": {"type": "int"},
                        "esm": {"type": "int"},
                        "trap": {"type": "int"},
                    },
                },
                "rate_limit": {
                    "type": "dict",
                    "options": {
                        "size": {"type": "int", "required": True},
                        "all": {"type": "bool"},
                        "console": {"type": "bool"},
                        "except_severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                    },
                },
                "reload": {
                    "type": "dict",
                    "options": {
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                        "message_limit": {"type": "int"},
                    },
                },
                "server_arp": {"type": "bool"},
                "snmp_trap": {
                    "type": "list",
                    "elements": "str",
                    "choices": [
                        "alerts",
                        "critical",
                        "debugging",
                        "emergencies",
                        "errors",
                        "informational",
                        "notifications",
                        "warnings",
                    ],
                },
                "source_interface": {
                    "type": "list",
                    "elements": "dict",
                    "options": {"interface": {"type": "str"}, "vrf": {"type": "str"}},
                },
                "trap": {
                    "type": "str",
                    "choices": [
                        "alerts",
                        "critical",
                        "debugging",
                        "emergencies",
                        "errors",
                        "informational",
                        "notifications",
                        "warnings",
                    ],
                },
                "userinfo": {"type": "bool"},
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "parsed",
                "rendered",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
"""
The module file for ios_lacp_interfaces
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: ios_lacp_interfaces
short_description: Resource module to configure LACP interfaces.
description:
  This module provides declarative management of LACP on Cisco IOS network
  devices lacp_interfaces.
version_added: 1.0.0
author: Sumit Jaiswal (@justjais)
notes:
  - Tested against Cisco IOSXE Version 17.3 on CML.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
options:
  config:
    description: A dictionary of LACP lacp_interfaces option
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Name of the Interface for configuring LACP.
        type: str
        required: true
      port_priority:
        description:
          - LACP priority on this interface.
          - Refer to vendor documentation for valid port values.
        type: int
      fast_switchover:
        description:
          - LACP fast switchover supported on this port channel.
        type: bool
      max_bundle:
        description:
          - LACP maximum number of ports to bundle in this port channel.
          - Refer to vendor documentation for valid port values.
        type: int
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by
        executing the command B(show running-config | section ^interface).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config
        | include ip route|ipv6 route) executed on device. For state I(parsed) active
        connection to remote host is not required.
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - parsed
    default: merged
"""

EXAMPLES = """
# Using merged
#
# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
# interface Port-channel20
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
# interface GigabitEthernet0/2
#  shutdown
# interface GigabitEthernet0/3
#  shutdown

- name: Merge provided configuration with device configuration
  cisco.ios.ios_lacp_interfaces:
    config:
      - name: GigabitEthernet0/1
        port_priority: 10
      - name: GigabitEthernet0/2
        port_priority: 20
      - name: GigabitEthernet0/3
        port_priority: 30
      - name: Port-channel10
        fast_switchover: true
        max_bundle: 5
    state: merged

# After state:
# ------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
#  lacp fast-switchover
#  lacp max-bundle 5
# interface Port-channel20
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
#  lacp port-priority 10
# interface GigabitEthernet0/2
#  shutdown
#  lacp port-priority 20
# interface GigabitEthernet0/3
#  shutdown
#  lacp port-priority 30

# Using overridden
#
# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
#  lacp fast-switchover
# interface Port-channel20
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
#  lacp port-priority 10
# interface GigabitEthernet0/2
#  shutdown
#  lacp port-priority 20
# interface GigabitEthernet0/3
#  shutdown
#  lacp port-priority 30

- name: Override device configuration of all lacp_interfaces with provided configuration
  cisco.ios.ios_lacp_interfaces:
    config:
      - name: GigabitEthernet0/1
        port_priority: 20
      - name: Port-channel10
        max_bundle: 2
    state: overridden

# After state:
# ------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
#  lacp max-bundle 2
# interface Port-channel20
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
#  lacp port-priority 20
# interface GigabitEthernet0/2
#  shutdown
# interface GigabitEthernet0/3
#  shutdown

# Using replaced
#
# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
#  lacp max-bundle 5
# interface Port-channel20
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
#  lacp port-priority 10
# interface GigabitEthernet0/2
#  shutdown
#  lacp port-priority 20
# interface GigabitEthernet0/3
#  shutdown
#  lacp port-priority 30

- name: Replaces device configuration of listed lacp_interfaces with provided configuration
  cisco.ios.ios_lacp_interfaces:
    config:
      - name: GigabitEthernet0/3
        port_priority: 40
      - name: Port-channel10
        fast_switchover: true
        max_bundle: 2
    state: replaced

# After state:
# ------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
#  lacp fast-switchover
#  lacp max-bundle 2
# interface Port-channel20
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
#  lacp port-priority 10
# interface GigabitEthernet0/2
#  shutdown
#  lacp port-priority 20
# interface GigabitEthernet0/3
#  shutdown
#  lacp port-priority 40

# Using Deleted
#
# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
#  flowcontrol receive on
# interface Port-channel20
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
#  lacp port-priority 10
# interface GigabitEthernet0/2
#  shutdown
#  lacp port-priority 20
# interface GigabitEthernet0/3
#  shutdown
#  lacp port-priority 30

- name: "Delete LACP attributes of given interfaces (Note: This won't delete the interface itself)"
  cisco.ios.ios_lacp_interfaces:
    config:
      - name: GigabitEthernet0/1
    state: deleted

# After state:
# -------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
# interface Port-channel20
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
# interface GigabitEthernet0/2
#  shutdown
#  lacp port-priority 20
# interface GigabitEthernet0/3
#  shutdown
#  lacp port-priority 30

# Using Deleted without any config passed
# "(NOTE: This will delete all of configured LLDP module attributes)"
#
# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
#  lacp fast-switchover
# interface Port-channel20
#  lacp max-bundle 2
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
#  lacp port-priority 10
# interface GigabitEthernet0/2
#  shutdown
#  lacp port-priority 20
# interface GigabitEthernet0/3
#  shutdown
#  lacp port-priority 30

- name: "Delete LACP attributes for all configured interfaces (Note: This won't delete the interface itself)"
  cisco.ios.ios_lacp_interfaces:
    state: deleted

# After state:
# -------------
#
# vios#show running-config | section ^interface
# interface Port-channel10
# interface Port-channel20
# interface Port-channel30
# interface GigabitEthernet0/1
#  shutdown
# interface GigabitEthernet0/2
#  shutdown
# interface GigabitEthernet0/3
#  shutdown

# Using Gathered

# Before state:
# -------------
#
# vios#sh running-config | section ^interface
# interface Port-channel10
#  lacp fast-switchover
#  lacp max-bundle 2
# interface Port-channel40
#  lacp max-bundle 5
# interface GigabitEthernet0/0
# interface GigabitEthernet0/1
#  lacp port-priority 30
# interface GigabitEthernet0/2
#  lacp port-priority 20

- name: Gather listed LACP interfaces with provided configurations
  cisco.ios.ios_lacp_interfaces:
    config:
    state: gathered

# Module Execution Result:
# ------------------------
#
# "gathered": [
#         {
#             "fast_switchover": true,
#             "max_bundle": 2,
#             "name": "Port-channel10"
#         },
#         {
#             "max_bundle": 5,
#             "name": "Port-channel40"
#         },
#         {
#             "name": "GigabitEthernet0/0"
#         },
#         {
#             "name": "GigabitEthernet0/1",
#             "port_priority": 30
#         },
#         {
#             "name": "GigabitEthernet0/2",
#             "port_priority": 20
#         }
#     ]

# After state:
# ------------
#
# vios#sh running-config | section ^interface
# interface Port-channel10
#  lacp fast-switchover
#  lacp max-bundle 2
# interface Port-channel40
#  lacp max-bundle 5
# interface GigabitEthernet0/0
# interface GigabitEthernet0/1
#  lacp port-priority 30
# interface GigabitEthernet0/2
#  lacp port-priority 20

# Using Rendered

- name: Render the commands for provided  configuration
  cisco.ios.ios_lacp_interfaces:
    config:
      - name: GigabitEthernet0/1
        port_priority: 10
      - name: GigabitEthernet0/2
        port_priority: 20
      - name: Port-channel10
        fast_switchover: true
        max_bundle: 2
    state: rendered

# Module Execution Result:
# ------------------------
#
# "rendered": [
#         "interface GigabitEthernet0/1",
#         "lacp port-priority 10",
#         "interface GigabitEthernet0/2",
#         "lacp port-priority 20",
#         "interface Port-channel10",
#         "lacp max-bundle 2",
#         "lacp fast-switchover"
#     ]

# Using Parsed

# File: parsed.cfg
# ----------------
#
# interface GigabitEthernet0/1
# lacp port-priority 10
# interface GigabitEthernet0/2
# lacp port-priority 20
# interface Port-channel10
# lacp max-bundle 2 fast-switchover

- name: Parse the commands for provided configuration
  cisco.ios.ios_lacp_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Module Execution Result:
# ------------------------
#
# "parsed": [
#         {
#             "name": "GigabitEthernet0/1",
#             "port_priority": 10
#         },
#         {
#             "name": "GigabitEthernet0/2",
#             "port_priority": 20
#         },
#         {
#             "fast_switchover": true,
#             "max_bundle": 2,
#             "name": "Port-channel10"
#         }
#     ]
"""

RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['interface GigabitEthernet 0/1', 'lacp port-priority 30']
"""
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.lacp_interfaces.lacp_interfaces import (
    Lacp_InterfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.lacp_interfaces.lacp_interfaces import (
    Lacp_Interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]

    module = AnsibleModule(
        argument_spec=Lacp_InterfacesArgs.argument_spec,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )
    result = Lacp_Interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2020, Tyler Auerbeck <tauerbec@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = r"""
---
module: digital_ocean_project_info
short_description: Gather information about DigitalOcean Projects
description:
    - This module can be used to gather information about Projects.
author: "Tyler Auerbeck (@tylerauerbeck)"
version_added: 1.6.0

options:
  id:
    description:
      - Project ID that can be used to identify and reference a project.
    type: str
  name:
    description:
      - Project name that can be used to identify and reference a project.
    type: str

extends_documentation_fragment:
- community.digitalocean.digital_ocean
"""


EXAMPLES = r"""
# Get specific project by id
- community.digitalocean.digital_ocean_project_info:
    id: cb1ef55e-3cd8-4c7c-aa5d-07c32bf41627

# Get specific project by name
- community.digitalocean.digital_ocean_project_info:
    name: my-project-name

# Get all projects
- community.digitalocean.digital_ocean_project_info:
  register: projects
"""

RETURN = r"""
data:
  description: "DigitalOcean project information"
  elements: dict
  returned: success
  type: list
  sample:
    - created_at: "2021-03-11T00:00:00Z"
      description: "My project description"
      environment: "Development"
      id: "12345678-abcd-efgh-5678-10111213"
      is_default: false
      name: "my-test-project"
      owner_id: 12345678
      owner_uuid: "12345678-1234-4321-abcd-20212223"
      purpose: ""
      updated_at: "2021-03-11T00:00:00Z"
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
)


def run(module):
    rest = DigitalOceanHelper(module)

    if module.params["id"]:
        response = rest.get("projects/{0}".format(module.params["id"]))
        if response.status_code != 200:
            module.fail_json(
                msg="Failed to fetch 'projects' information due to error: %s"
                % response.json["message"]
            )
    else:
        response = rest.get_paginated_data(
            base_url="projects?", data_key_name="projects"
        )

    if module.params["id"]:
        data = [response.json["project"]]
    elif module.params["name"]:
        data = [d for d in response if d["name"] == module.params["name"]]
        if not data:
            module.fail_json(
                msg="Failed to fetch 'projects' information due to error: Unable to find project with name %s"
                % module.params["name"]
            )
    else:
        data = response

    module.exit_json(changed=False, data=data)


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        name=dict(type="str", required=False, default=None),
        id=dict(type="str", required=False, default=None),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[("id", "name")],
    )
    run(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_devtestlabartifactsource
version_added: "0.1.2"
short_description: Manage Azure DevTest Labs Artifacts Source instance
description:
    - Create, update and delete instance of Azure DevTest Labs Artifacts Source.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: True
    lab_name:
        description:
            - The name of the lab.
        type: str
        required: True
    name:
        description:
            - The name of the artifact source.
        type: str
        required: True
    display_name:
        description:
            - The artifact source's display name.
        type: str
    uri:
        description:
            - The artifact source's URI.
        type: str
    source_type:
        description:
            - The artifact source's type.
        type: str
        choices:
            - 'vso'
            - 'github'
    folder_path:
        description:
            - The folder containing artifacts.
        type: str
    arm_template_folder_path:
        description:
            - The folder containing Azure Resource Manager templates.
        type: str
    branch_ref:
        description:
            - The artifact source's branch reference.
        type: str
    security_token:
        description:
            - The security token to authenticate to the artifact source.
        type: str
    is_enabled:
        description:
            - Indicates whether the artifact source is enabled.
        type: bool
    state:
        description:
            - Assert the state of the DevTest Labs Artifacts Source.
            - Use C(present) to create or update an DevTest Labs Artifacts Source and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Create (or update) DevTest Labs Artifacts Source
  azure_rm_devtestlabartifactsource:
    resource_group: myrg
    lab_name: mylab
    name: myartifacts
    uri: https://github.com/myself/myrepo.git
    source_type: github
    folder_path: /
    security_token: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
'''

RETURN = '''
id:
    description:
        - The identifier of the resource.
    returned: always
    type: str
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myrg/providers/microsoft.devtestlab/labs/mylab/artifactsources/myartifacts
is_enabled:
    description:
        - Indicates whether the artifact source is enabled.
    returned: always
    type: bool
    sample: true
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.devtestlabs import DevTestLabsClient
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMDevTestLabArtifactsSource(AzureRMModuleBase):
    """Configuration class for an Azure RM DevTest Labs Artifacts Source resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            lab_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            display_name=dict(
                type='str'
            ),
            uri=dict(
                type='str'
            ),
            source_type=dict(
                type='str',
                choices=['vso',
                         'github']
            ),
            folder_path=dict(
                type='str'
            ),
            arm_template_folder_path=dict(
                type='str'
            ),
            branch_ref=dict(
                type='str'
            ),
            security_token=dict(
                type='str',
                no_log=True
            ),
            is_enabled=dict(
                type='bool'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.lab_name = None
        self.name = None
        self.artifact_source = dict()

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction

        required_if = [
            ('state', 'present', [
             'source_type', 'uri', 'security_token'])
        ]

        super(AzureRMDevTestLabArtifactsSource, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                               supports_check_mode=True,
                                                               supports_tags=True,
                                                               required_if=required_if)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.artifact_source[key] = kwargs[key]

        if self.artifact_source.get('source_type') == 'github':
            self.artifact_source['source_type'] = 'GitHub'
        elif self.artifact_source.get('source_type') == 'vso':
            self.artifact_source['source_type'] = 'VsoGit'

        if self.artifact_source.get('status') is not None:
            self.artifact_source['status'] = 'Enabled' if self.artifact_source.get('status') else 'Disabled'

        response = None

        self.mgmt_client = self.get_mgmt_svc_client(DevTestLabsClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager,
                                                    is_track2=True,
                                                    api_version='2018-10-15')

        old_response = self.get_devtestlabartifactssource()

        if not old_response:
            self.log("DevTest Labs Artifacts Source instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("DevTest Labs Artifacts Source instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                self.results['old_response'] = old_response

                if self.artifact_source.get('display_name') is not None:
                    if self.artifact_source.get('display_name') != old_response.get('display_name'):
                        self.to_do = Actions.Update
                else:
                    self.artifact_source['display_name'] = old_response.get('display_name')

                if self.artifact_source.get('source_type').lower() != old_response.get('source_type').lower():
                    self.to_do = Actions.Update

                if self.artifact_source.get('uri') != old_response.get('uri'):
                    self.to_do = Actions.Update

                if self.artifact_source.get('branch_ref') is not None:
                    if self.artifact_source.get('branch_ref') != old_response.get('branch_ref'):
                        self.to_do = Actions.Update
                else:
                    self.artifact_source['branch_ref'] = old_response.get('branch_ref')

                if self.artifact_source.get('status') is not None:
                    if self.artifact_source.get('status') != old_response.get('status'):
                        self.to_do = Actions.Update
                else:
                    self.artifact_source['status'] = old_response.get('status')

                if self.artifact_source.get('folder_path') is not None:
                    if self.artifact_source.get('folder_path') != old_response.get('folder_path'):
                        self.to_do = Actions.Update
                else:
                    self.artifact_source['folder_path'] = old_response.get('folder_path')

                if self.artifact_source.get('arm_template_folder_path') is not None:
                    if self.artifact_source.get('arm_template_folder_path') != old_response.get('arm_template_folder_path'):
                        self.to_do = Actions.Update
                else:
                    self.artifact_source['arm_template_folder_path'] = old_response.get('arm_template_folder_path')

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the DevTest Labs Artifacts Source instance")
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.create_update_devtestlabartifactssource()
            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("DevTest Labs Artifacts Source instance deleted")
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            self.delete_devtestlabartifactssource()
        else:
            self.log("DevTest Labs Artifacts Source instance unchanged")
            self.results['changed'] = False
            response = old_response

        if self.state == 'present':
            self.results.update({
                'id': response.get('id', None),
                'is_enabled': (response.get('status', None).lower() == 'enabled')
            })
        return self.results

    def create_update_devtestlabartifactssource(self):
        '''
        Creates or updates DevTest Labs Artifacts Source with the specified configuration.

        :return: deserialized DevTest Labs Artifacts Source instance state dictionary
        '''
        self.log("Creating / Updating the DevTest Labs Artifacts Source instance {0}".format(self.name))

        try:
            response = self.mgmt_client.artifact_sources.create_or_update(resource_group_name=self.resource_group,
                                                                          lab_name=self.lab_name,
                                                                          name=self.name,
                                                                          artifact_source=self.artifact_source)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the DevTest Labs Artifacts Source instance.')
            self.fail("Error creating the DevTest Labs Artifacts Source instance: {0}".format(str(exc)))
        return response.as_dict()

    def delete_devtestlabartifactssource(self):
        '''
        Deletes specified DevTest Labs Artifacts Source instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the DevTest Labs Artifacts Source instance {0}".format(self.name))
        try:
            response = self.mgmt_client.artifact_sources.delete(resource_group_name=self.resource_group,
                                                                lab_name=self.lab_name,
                                                                name=self.name)
        except Exception as e:
            self.log('Error attempting to delete the DevTest Labs Artifacts Source instance.')
            self.fail("Error deleting the DevTest Labs Artifacts Source instance: {0}".format(str(e)))

        return True

    def get_devtestlabartifactssource(self):
        '''
        Gets the properties of the specified DevTest Labs Artifacts Source.

        :return: deserialized DevTest Labs Artifacts Source instance state dictionary
        '''
        self.log("Checking if the DevTest Labs Artifacts Source instance {0} is present".format(self.name))
        found = False
        try:
            response = self.mgmt_client.artifact_sources.get(resource_group_name=self.resource_group,
                                                             lab_name=self.lab_name,
                                                             name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("DevTest Labs Artifacts Source instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the DevTest Labs Artifacts Source instance.')
        if found is True:
            return response.as_dict()

        return False


def main():
    """Main execution"""
    AzureRMDevTestLabArtifactsSource()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: meraki_ms_switchport
short_description: Manage switchports on a switch in the Meraki cloud
description:
- Allows for management of switchports settings for Meraki MS switches.
deprecated:
  removed_in: '3.0.0'
  why: Updated modules released with increased functionality
  alternative: cisco.meraki.devices_switch_ports
options:
    state:
        description:
        - Specifies whether a switchport should be queried or modified.
        choices: [query, present]
        default: query
        type: str
    access_policy_type:
        description:
        - Type of access policy to apply to port.
        type: str
        choices: [Open, Custom access policy, MAC allow list, Sticky MAC allow list]
    access_policy_number:
        description:
        - Number of the access policy to apply.
        - Only applicable to access port types.
        type: int
    allowed_vlans:
        description:
        - List of VLAN numbers to be allowed on switchport.
        default: all
        type: list
        elements: str
    enabled:
        description:
        - Whether a switchport should be enabled or disabled.
        type: bool
        default: yes
    isolation_enabled:
        description:
        - Isolation status of switchport.
        default: no
        type: bool
    link_negotiation:
        description:
        - Link speed for the switchport.
        default: Auto negotiate
        choices: [1 Gigabit full duplex (auto),
                1 Gigabit full duplex (forced),
                10 Gigabit full duplex (auto),
                10 Gigabit full duplex (forced),
                100 Megabit (auto),
                100 Megabit full duplex (forced),
                2.5 Gigabit full duplex (auto),
                2.5 Gigabit full duplex (forced),
                5 Gigabit full duplex (auto),
                5 Gigabit full duplex (forced),
                Auto negotiate]
        type: str
    name:
        description:
        - Switchport description.
        aliases: [description]
        type: str
    number:
        description:
        - Port number.
        type: str
    poe_enabled:
        description:
        - Enable or disable Power Over Ethernet on a port.
        type: bool
        default: true
    rstp_enabled:
        description:
        - Enable or disable Rapid Spanning Tree Protocol on a port.
        type: bool
        default: true
    serial:
        description:
        - Serial nubmer of the switch.
        type: str
        required: true
    stp_guard:
        description:
        - Set state of STP guard.
        choices: [disabled, root guard, bpdu guard, loop guard]
        default: disabled
        type: str
    tags:
        description:
        - List of tags to assign to a port.
        type: list
        elements: str
    type:
        description:
        - Set port type.
        choices: [access, trunk]
        default: access
        type: str
    vlan:
        description:
        - VLAN number assigned to port.
        - If a port is of type trunk, the specified VLAN is the native VLAN.
        - Setting value to 0 on a trunk will clear the VLAN.
        type: int
    voice_vlan:
        description:
        - VLAN number assigned to a port for voice traffic.
        - Only applicable to access port type.
        - Only applicable if voice_vlan_state is set to present.
        type: int
    voice_vlan_state:
        description:
        - Specifies whether voice vlan configuration should be present or absent.
        choices: [absent, present]
        default: present
        type: str
    mac_allow_list:
        description:
        - MAC addresses list that are allowed on a port.
        - Only applicable to access port type.
        - Only applicable to access_policy_type "MAC allow list".
        type: dict
        suboptions:
            state:
                description:
                - The state the configuration should be left in.
                - Merged, MAC addresses provided will be added to the current allow list.
                - Replaced, All MAC addresses are overwritten, only the MAC addresses provided with exist in the allow list.
                - Deleted, Remove the MAC addresses provided from the current allow list.
                type: str
                choices: [merged, replaced, deleted]
                default: replaced
            macs:
                description:
                - List of MAC addresses to update with based on state option.
                type: list
                elements: str
    sticky_mac_allow_list:
        description:
        - MAC addresses list that are allowed on a port.
        - Only applicable to access port type.
        - Only applicable to access_policy_type "Sticky MAC allow list".
        type: dict
        suboptions:
            state:
                description:
                - The state the configuration should be left in.
                - Merged, MAC addresses provided will be added to the current allow list.
                - Replaced, All MAC addresses are overwritten, only the MAC addresses provided with exist in the allow list.
                - Deleted, Remove the MAC addresses provided from the current allow list.
                type: str
                choices: ["merged", "replaced", "deleted"]
                default: replaced
            macs:
                description:
                - List of MAC addresses to update with based on state option.
                type: list
                elements: str
    sticky_mac_allow_list_limit:
        description:
        - The number of MAC addresses allowed in the sticky port allow list.
        - Only applicable to access port type.
        - Only applicable to access_policy_type "Sticky MAC allow list".
        - The value must be equal to or greater then the list size of sticky_mac_allow_list. Value will be checked for validity, during processing.
        type: int
    flexible_stacking_enabled:
        description:
        - Whether flexible stacking capabilities are supported on the port.
        type: bool
author:
- Kevin Breit (@kbreit)
extends_documentation_fragment: cisco.meraki.meraki
"""

EXAMPLES = r"""
- name: Query information about all switchports on a switch
  meraki_switchport:
    auth_key: abc12345
    state: query
    serial: ABC-123
  delegate_to: localhost

- name: Query information about all switchports on a switch
  meraki_switchport:
    auth_key: abc12345
    state: query
    serial: ABC-123
    number: 2
  delegate_to: localhost

- name: Name switchport
  meraki_switchport:
    auth_key: abc12345
    state: present
    serial: ABC-123
    number: 7
    name: Test Port
  delegate_to: localhost

- name: Configure access port with voice VLAN
  meraki_switchport:
    auth_key: abc12345
    state: present
    serial: ABC-123
    number: 7
    enabled: true
    name: Test Port
    tags: desktop
    type: access
    vlan: 10
    voice_vlan: 11
  delegate_to: localhost

- name: Check access port for idempotency
  meraki_switchport:
    auth_key: abc12345
    state: present
    serial: ABC-123
    number: 7
    enabled: true
    name: Test Port
    tags: desktop
    type: access
    vlan: 10
    voice_vlan: 11
  delegate_to: localhost

- name: Configure trunk port with specific VLANs
  meraki_switchport:
    auth_key: abc12345
    state: present
    serial: ABC-123
    number: 7
    enabled: true
    name: Server port
    tags: server
    type: trunk
    allowed_vlans:
      - 10
      - 15
      - 20
  delegate_to: localhost

- name: Configure access port with sticky MAC allow list and limit.
  meraki_switchport:
    auth_key: abc12345
    state: present
    serial: ABC-123
    number: 5
    sticky_mac_allow_limit: 3
    sticky_mac_allow_list:
        macs:
          - aa:aa:bb:bb:cc:cc
          - bb:bb:aa:aa:cc:cc
          - 11:aa:bb:bb:cc:cc
        state: replaced
    delegate_to: localhost

- name: Delete an existing MAC address from the sticky MAC allow list.
  meraki_switchport:
    auth_key: abc12345
    state: present
    serial: ABC-123
    number: 5
    sticky_mac_allow_list:
        macs:
          - aa:aa:bb:bb:cc:cc
        state: deleted
    delegate_to: localhost

- name: Add a MAC address to sticky MAC allow list.
  meraki_switchport:
    auth_key: abc12345
    state: present
    serial: ABC-123
    number: 5
    sticky_mac_allow_list:
        macs:
          - 22:22:bb:bb:cc:cc
        state: merged
    delegate_to: localhost
"""

RETURN = r"""
data:
    description: Information queried or updated switchports.
    returned: success
    type: complex
    contains:
        access_policy_type:
            description: Type of access policy assigned to port
            returned: success, when assigned
            type: str
            sample: "MAC allow list"
        allowed_vlans:
            description: List of VLANs allowed on an access port
            returned: success, when port is set as access
            type: str
            sample: all
        number:
            description: Number of port.
            returned: success
            type: int
            sample: 1
        name:
            description: Human friendly description of port.
            returned: success
            type: str
            sample: "Jim Phone Port"
        tags:
            description: List of tags assigned to port.
            returned: success
            type: list
            sample: ['phone', 'marketing']
        enabled:
            description: Enabled state of port.
            returned: success
            type: bool
            sample: true
        poe_enabled:
            description: Power Over Ethernet enabled state of port.
            returned: success
            type: bool
            sample: true
        type:
            description: Type of switchport.
            returned: success
            type: str
            sample: trunk
        vlan:
            description: VLAN assigned to port.
            returned: success
            type: int
            sample: 10
        voice_vlan:
            description: VLAN assigned to port with voice VLAN enabled devices.
            returned: success
            type: int
            sample: 20
        isolation_enabled:
            description: Port isolation status of port.
            returned: success
            type: bool
            sample: true
        rstp_enabled:
            description: Enabled or disabled state of Rapid Spanning Tree Protocol (RSTP)
            returned: success
            type: bool
            sample: true
        stp_guard:
            description: State of STP guard
            returned: success
            type: str
            sample: "Root Guard"
        access_policy_number:
            description: Number of assigned access policy. Only applicable to access ports.
            returned: success
            type: int
            sample: 1234
        link_negotiation:
            description: Link speed for the port.
            returned: success
            type: str
            sample: "Auto negotiate"
        sticky_mac_allow_list_limit:
            description: Number of MAC addresses allowed on a sticky port.
            returned: success
            type: int
            sample: 6
        sticky_mac_allow_list:
            description: List of MAC addresses currently allowed on a sticky port. Used with access_policy_type of Sticky MAC allow list.
            returned: success
            type: list
            sample: ["11:aa:bb:bb:cc:cc", "22:aa:bb:bb:cc:cc", "33:aa:bb:bb:cc:cc"]
        mac_allow_list:
            description: List of MAC addresses currently allowed on a non-sticky port. Used with access_policy_type of MAC allow list.
            returned: success
            type: list
            sample: ["11:aa:bb:bb:cc:cc", "22:aa:bb:bb:cc:cc", "33:aa:bb:bb:cc:cc"]
        port_schedule_id:
            description: Unique ID of assigned port schedule
            returned: success
            type: str
            sample: null
        udld:
            description: Alert state of UDLD
            returned: success
            type: str
            sample: "Alert only"
        flexible_stacking_enabled:
            description: Whether flexible stacking capabilities are enabled on the port.
            returned: success
            type: bool
"""

from ansible.module_utils.basic import AnsibleModule, json
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import (
    MerakiModule,
    meraki_argument_spec,
)

param_map = {
    "access_policy_number": "accessPolicyNumber",
    "access_policy_type": "accessPolicyType",
    "allowed_vlans": "allowedVlans",
    "enabled": "enabled",
    "isolation_enabled": "isolationEnabled",
    "link_negotiation": "linkNegotiation",
    "name": "name",
    "number": "number",
    "poe_enabled": "poeEnabled",
    "rstp_enabled": "rstpEnabled",
    "stp_guard": "stpGuard",
    "tags": "tags",
    "type": "type",
    "vlan": "vlan",
    "voice_vlan": "voiceVlan",
    "mac_allow_list": "macAllowList",
    "sticky_mac_allow_list": "stickyMacAllowList",
    "sticky_mac_allow_list_limit": "stickyMacAllowListLimit",
    "adaptive_policy_group_id": "adaptivePolicyGroupId",
    "peer_sgt_capable": "peerSgtCapable",
    "flexible_stacking_enabled": "flexibleStackingEnabled",
}


def sort_vlans(meraki, vlans):
    converted = set()
    for vlan in vlans:
        converted.add(int(vlan))
    vlans_sorted = sorted(converted)
    vlans_str = []
    for vlan in vlans_sorted:
        vlans_str.append(str(vlan))
    return ",".join(vlans_str)


def assemble_payload(meraki):
    payload = dict()
    # if meraki.params['enabled'] is not None:
    #     payload['enabled'] = meraki.params['enabled']

    for k, v in meraki.params.items():
        try:
            if meraki.params[k] is not None:
                if k == "access_policy_number":
                    if meraki.params["access_policy_type"] is not None:
                        payload[param_map[k]] = v
                else:
                    payload[param_map[k]] = v
        except KeyError:
            pass
    return payload


def get_mac_list(original_allowed, new_mac_list, state):
    if state == "deleted":
        return [entry for entry in original_allowed if entry not in new_mac_list]
    if state == "merged":
        return original_allowed + list(set(new_mac_list) - set(original_allowed))
    return new_mac_list


def clear_vlan(params, payload):
    if params["vlan"] == 0:
        payload["vlan"] = None
    return payload


def main():
    # define the available arguments/parameters that a user can pass to
    # the module
    argument_spec = meraki_argument_spec()

    policy_data_arg_spec = dict(
        macs=dict(type="list", elements="str"),
        state=dict(
            type="str", choices=["merged", "replaced", "deleted"], default="replaced"
        ),
    )

    argument_spec.update(
        state=dict(type="str", choices=["present", "query"], default="query"),
        serial=dict(type="str", required=True),
        number=dict(type="str"),
        name=dict(type="str", aliases=["description"]),
        tags=dict(type="list", elements="str"),
        enabled=dict(type="bool", default=True),
        type=dict(type="str", choices=["access", "trunk"], default="access"),
        vlan=dict(type="int"),
        voice_vlan=dict(type="int"),
        voice_vlan_state=dict(
            type="str", choices=["present", "absent"], default="present"
        ),
        allowed_vlans=dict(type="list", elements="str", default="all"),
        poe_enabled=dict(type="bool", default=True),
        isolation_enabled=dict(type="bool", default=False),
        rstp_enabled=dict(type="bool", default=True),
        stp_guard=dict(
            type="str",
            choices=["disabled", "root guard", "bpdu guard", "loop guard"],
            default="disabled",
        ),
        access_policy_type=dict(
            type="str",
            choices=[
                "Open",
                "Custom access policy",
                "MAC allow list",
                "Sticky MAC allow list",
            ],
        ),
        access_policy_number=dict(type="int"),
        link_negotiation=dict(
            type="str",
            choices=[
                "1 Gigabit full duplex (auto)",
                "1 Gigabit full duplex (forced)",
                "10 Gigabit full duplex (auto)",
                "10 Gigabit full duplex (forced)",
                "100 Megabit (auto)",
                "100 Megabit full duplex (forced)",
                "2.5 Gigabit full duplex (auto)",
                "2.5 Gigabit full duplex (forced)",
                "5 Gigabit full duplex (auto)",
                "5 Gigabit full duplex (forced)",
                "Auto negotiate",
            ],
            default="Auto negotiate",
        ),
        mac_allow_list=dict(type="dict", options=policy_data_arg_spec),
        sticky_mac_allow_list=dict(type="dict", options=policy_data_arg_spec),
        sticky_mac_allow_list_limit=dict(type="int"),
        # adaptive_policy_group_id=dict(type=str),
        # peer_sgt_capable=dict(type=bool),
        flexible_stacking_enabled=dict(type="bool"),
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    meraki = MerakiModule(module, function="switchport")
    if meraki.params.get("voice_vlan_state") == "absent" and meraki.params.get(
        "voice_vlan"
    ):
        meraki.fail_json(
            msg="voice_vlan_state cant be `absent` while voice_vlan is also defined."
        )

    meraki.params["follow_redirects"] = "all"

    if meraki.params["type"] == "trunk":
        if not meraki.params["allowed_vlans"]:
            meraki.params["allowed_vlans"] = [
                "all"
            ]  # Backdoor way to set default without conflicting on access

    query_urls = {"switchport": "/devices/{serial}/switch/ports"}
    query_url = {"switchport": "/devices/{serial}/switch/ports/{number}"}
    update_url = {"switchport": "/devices/{serial}/switch/ports/{number}"}

    meraki.url_catalog["get_all"].update(query_urls)
    meraki.url_catalog["get_one"].update(query_url)
    meraki.url_catalog["update"] = update_url

    # execute checks for argument completeness

    # manipulate or modify the state as needed (this is going to be the
    # part where your module will do what it needs to do)
    if meraki.params["state"] == "query":
        if meraki.params["number"]:
            path = meraki.construct_path(
                "get_one",
                custom={
                    "serial": meraki.params["serial"],
                    "number": meraki.params["number"],
                },
            )
            response = meraki.request(path, method="GET")
            meraki.result["data"] = response
        else:
            path = meraki.construct_path(
                "get_all", custom={"serial": meraki.params["serial"]}
            )
            response = meraki.request(path, method="GET")
            meraki.result["data"] = response
    elif meraki.params["state"] == "present":
        payload = assemble_payload(meraki)
        # meraki.fail_json(msg='payload', payload=payload)
        allowed = set()  # Use a set to remove duplicate items
        if meraki.params["allowed_vlans"][0] == "all":
            allowed.add("all")
        else:
            for vlan in meraki.params["allowed_vlans"]:
                allowed.add(str(vlan))
            if meraki.params["vlan"] is not None:
                allowed.add(str(meraki.params["vlan"]))
        if len(allowed) > 1:  # Convert from list to comma separated
            payload["allowedVlans"] = sort_vlans(meraki, allowed)
        else:
            payload["allowedVlans"] = next(iter(allowed))

        # Exceptions need to be made for idempotency check based on how Meraki returns
        if meraki.params["type"] == "access":
            if not meraki.params[
                "vlan"
            ]:  # VLAN needs to be specified in access ports, but can't default to it
                payload["vlan"] = 1
        query_path = meraki.construct_path(
            "get_one",
            custom={
                "serial": meraki.params["serial"],
                "number": meraki.params["number"],
            },
        )
        original = meraki.request(query_path, method="GET")
        # Check voiceVlan to see if state is absent to remove the vlan.
        if meraki.params.get("voice_vlan_state"):
            if meraki.params.get("voice_vlan_state") == "absent":
                payload["voiceVlan"] = None
            else:
                payload["voiceVlan"] = meraki.params.get("voice_vlan")
        if meraki.params.get("mac_allow_list"):
            macs = get_mac_list(
                original.get("macAllowList"),
                meraki.params["mac_allow_list"].get("macs"),
                meraki.params["mac_allow_list"].get("state"),
            )
            payload["macAllowList"] = macs
        # Evaluate Sticky Limit whether it was passed in or what is currently configured and was returned in GET call.
        if meraki.params.get("sticky_mac_allow_list_limit"):
            sticky_mac_limit = meraki.params.get("sticky_mac_allow_list_limit")
        else:
            sticky_mac_limit = original.get("stickyMacAllowListLimit")
        if meraki.params.get("sticky_mac_allow_list"):
            macs = get_mac_list(
                original.get("stickyMacAllowList"),
                meraki.params["sticky_mac_allow_list"].get("macs"),
                meraki.params["sticky_mac_allow_list"].get("state"),
            )
            if int(sticky_mac_limit) < len(macs):
                meraki.fail_json(
                    msg="Stick MAC Allow List Limit must be equal to or greater than length of Sticky MAC Allow List."
                )
            payload["stickyMacAllowList"] = macs
            payload["stickyMacAllowListLimit"] = sticky_mac_limit
        payload = clear_vlan(meraki.params, payload)
        proposed = payload.copy()
        if meraki.params["type"] == "trunk":
            proposed["voiceVlan"] = original[
                "voiceVlan"
            ]  # API shouldn't include voice VLAN on a trunk port
        # meraki.fail_json(msg='Compare', original=original, payload=payload)
        if meraki.is_update_required(original, proposed, optional_ignore=["number"]):
            if meraki.check_mode is True:
                original.update(proposed)
                meraki.result["data"] = original
                meraki.result["changed"] = True
                meraki.exit_json(**meraki.result)
            path = meraki.construct_path(
                "update",
                custom={
                    "serial": meraki.params["serial"],
                    "number": meraki.params["number"],
                },
            )
            response = meraki.request(path, method="PUT", payload=json.dumps(payload))
            meraki.result["data"] = response
            meraki.result["changed"] = True
        else:
            meraki.result["data"] = original

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function


__metaclass__ = type

# (c) 2017, Ansible by Red Hat, inc
#
# This file is part of Ansible by Red Hat
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#


DOCUMENTATION = """
module: eos_vrf
author: Ricardo Carrillo Cruz (@rcarrillocruz)
short_description: Manage VRFs on Arista EOS network devices
description:
- This module provides declarative management of VRFs on Arista EOS network devices.
version_added: 1.0.0
notes:
- Tested against Arista EOS 4.24.6F
options:
  name:
    description:
    - Name of the VRF.
    type: str
  rd:
    description:
    - Route distinguisher of the VRF
    type: str
  interfaces:
    description:
    - Identifies the set of interfaces that should be configured in the VRF. Interfaces
      must be routed interfaces in order to be placed into a VRF. The name of interface
      should be in expanded format and not abbreviated.
    type: list
    elements: str
  associated_interfaces:
    description:
    - This is a intent option and checks the operational state of the for given vrf
      C(name) for associated interfaces. If the value in the C(associated_interfaces)
      does not match with the operational state of vrf interfaces on device it will
      result in failure.
    type: list
    elements: str
  aggregate:
    description: List of VRFs instances
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Name of the VRF.
        required: true
        type: str
      rd:
        description:
        - Route distinguisher of the VRF
        type: str
      interfaces:
        description:
        - Identifies the set of interfaces that should be configured in the VRF. Interfaces
          must be routed interfaces in order to be placed into a VRF. The name of interface
          should be in expanded format and not abbreviated.
        type: list
        elements: str
      associated_interfaces:
        description:
        - This is a intent option and checks the operational state of the for given vrf
          C(name) for associated interfaces. If the value in the C(associated_interfaces)
          does not match with the operational state of vrf interfaces on device it will
          result in failure.
        type: list
        elements: str
      delay:
        description:
        - Time in seconds to wait before checking for the operational state on remote
          device. This wait is applicable for operational state arguments.
        default: 10
        type: int
      state:
        description:
        - State of the VRF configuration.
        default: present
        type: str
        choices:
        - present
        - absent
  purge:
    description:
    - Purge VRFs not defined in the I(aggregate) parameter.
    default: false
    type: bool
  delay:
    description:
    - Time in seconds to wait before checking for the operational state on remote
      device. This wait is applicable for operational state arguments.
    default: 10
    type: int
  state:
    description:
    - State of the VRF configuration.
    default: present
    type: str
    choices:
    - present
    - absent
"""

EXAMPLES = """
- name: Create vrf
  arista.eos.eos_vrf:
    name: test
    rd: 1:200
    interfaces:
      - Ethernet2
    state: present

- name: Delete VRFs
  arista.eos.eos_vrf:
    name: test
    state: absent

- name: Create aggregate of VRFs with purge
  arista.eos.eos_vrf:
    aggregate:
      - name: test4
        rd: 1:204
      - name: test5
        rd: 1:205
    state: present
    purge: true

- name: Delete aggregate of VRFs
  arista.eos.eos_vrf:
    aggregate:
      - name: test2
      - name: test3
      - name: test4
      - name: test5
    state: absent
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - vrf instance test
    - rd 1:100
    - interface Ethernet1
    - vrf test
"""
import re
import time

from copy import deepcopy

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    remove_default_spec,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.eos import (
    load_config,
    run_commands,
)


def search_obj_in_list(name, lst):
    for o in lst:
        if o["name"] == name:
            return o


def map_obj_to_commands(updates, module):
    commands = list()
    want, have = updates
    state = module.params["state"]
    purge = module.params["purge"]

    for w in want:
        name = w["name"]
        rd = w["rd"]

        obj_in_have = search_obj_in_list(name, have)

        if state == "absent":
            if obj_in_have:
                commands.append("no vrf instance %s" % name)
        elif state == "present":
            if not obj_in_have:
                commands.append("vrf instance %s" % name)

                if rd is not None:
                    commands.append("rd %s" % rd)

                if w["interfaces"]:
                    for i in w["interfaces"]:
                        commands.append("interface %s" % i)
                        commands.append("vrf %s" % w["name"])
            else:
                if w["rd"] is not None and w["rd"] != obj_in_have["rd"]:
                    commands.append("vrf instance %s" % w["name"])
                    commands.append("rd %s" % w["rd"])

                if w["interfaces"]:
                    if not obj_in_have["interfaces"]:
                        for i in w["interfaces"]:
                            commands.append("interface %s" % i)
                            commands.append("vrf %s" % w["name"])
                    elif set(w["interfaces"]) != obj_in_have["interfaces"]:
                        missing_interfaces = list(
                            set(w["interfaces"]) - set(obj_in_have["interfaces"]),
                        )

                        for i in missing_interfaces:
                            commands.append("interface %s" % i)
                            commands.append("vrf %s" % w["name"])

    if purge:
        for h in have:
            obj_in_want = search_obj_in_list(h["name"], want)
            if not obj_in_want:
                commands.append("no vrf instance %s" % h["name"])

    return commands


def map_config_to_obj(module):
    objs = []
    output = run_commands(module, {"command": "show vrf", "output": "text"})

    lines = output[0].strip().splitlines()[3:]

    out_len = len(lines)
    index = 0
    while out_len > index:
        line = lines[index]
        if not line:
            continue

        splitted_line = re.split(r"\s{2,}", line.strip())

        if len(splitted_line) == 1:
            index += 1
            continue
        obj = dict()
        obj["name"] = splitted_line[0]
        obj["rd"] = splitted_line[1]
        obj["interfaces"] = []

        if len(splitted_line) > 4:
            obj["interfaces"] = []
            interfaces = splitted_line[4]
            if interfaces.endswith(","):
                while interfaces.endswith(","):
                    # gather all comma separated interfaces
                    if out_len <= index:
                        break
                    index += 1
                    line = lines[index]
                    vrf_line = re.split(r"\s{2,}", line.strip())
                    interfaces += vrf_line[-1]

            for i in interfaces.split(","):
                obj["interfaces"].append(i.strip().lower())
        index += 1
        objs.append(obj)

    return objs


def map_params_to_obj(module):
    obj = []
    aggregate = module.params.get("aggregate")
    if aggregate:
        for item in aggregate:
            for key in item:
                if item.get(key) is None:
                    item[key] = module.params[key]

            if item.get("interfaces"):
                item["interfaces"] = [
                    intf.replace(" ", "").lower() for intf in item.get("interfaces") if intf
                ]

            if item.get("associated_interfaces"):
                item["associated_interfaces"] = [
                    intf.replace(" ", "").lower()
                    for intf in item.get("associated_interfaces")
                    if intf
                ]

            obj.append(item.copy())
    else:
        obj.append(
            {
                "name": module.params["name"],
                "state": module.params["state"],
                "rd": module.params["rd"],
                "interfaces": [
                    intf.replace(" ", "").lower() for intf in module.params["interfaces"]
                ]
                if module.params["interfaces"]
                else [],
                "associated_interfaces": [
                    intf.replace(" ", "").lower() for intf in module.params["associated_interfaces"]
                ]
                if module.params["associated_interfaces"]
                else [],
            },
        )

    return obj


def check_declarative_intent_params(want, module, result):
    have = None
    is_delay = False

    for w in want:
        if w.get("associated_interfaces") is None:
            continue

        if result["changed"] and not is_delay:
            time.sleep(module.params["delay"])
            is_delay = True

        if have is None:
            have = map_config_to_obj(module)

        for i in w["associated_interfaces"]:
            obj_in_have = search_obj_in_list(w["name"], have)

            if obj_in_have:
                interfaces = obj_in_have.get("interfaces")
                if interfaces is not None and i not in interfaces:
                    module.fail_json(
                        msg="Interface %s not configured on vrf %s" % (i, w["name"]),
                    )


def main():
    """main entry point for module execution"""
    element_spec = dict(
        name=dict(),
        interfaces=dict(type="list", elements="str"),
        associated_interfaces=dict(type="list", elements="str"),
        delay=dict(default=10, type="int"),
        rd=dict(),
        state=dict(default="present", choices=["present", "absent"]),
    )

    aggregate_spec = deepcopy(element_spec)
    aggregate_spec["name"] = dict(required=True)

    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)
    aggregate_spec["state"].update(default="present")
    aggregate_spec["delay"].update(default=10)

    argument_spec = dict(
        aggregate=dict(type="list", elements="dict", options=aggregate_spec),
        purge=dict(default=False, type="bool"),
    )

    argument_spec.update(element_spec)

    required_one_of = [["name", "aggregate"]]
    mutually_exclusive = [["name", "aggregate"]]
    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=required_one_of,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    warnings = list()

    result = {"changed": False}

    if warnings:
        result["warnings"] = warnings

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)

    commands = map_obj_to_commands((want, have), module)
    result["commands"] = commands

    if commands:
        commit = not module.check_mode
        response = load_config(module, commands, commit=commit)
        if response.get("diff") and module._diff:
            result["diff"] = {"prepared": response.get("diff")}
        result["session_name"] = response.get("session")
        result["changed"] = True

    check_declarative_intent_params(want, module, result)

    module.exit_json(**result)


if __name__ == "__main__":
    main()
